
/*
 * @(#)Channel.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data Channel class.
 *
 * @version     0.94 3 April 1997
 * @author      Rich Burridge
 */

public
class Channel extends Manageable {

    /** To signify a manageable object of type Channel. */
    public static final String M_Channel = "Channel";

    /** The receivers of Data sent over a Channel. */
    public static final String D_All     = "All";
    public static final String D_Others  = "Others";

    /* The various priorities that data can be sent at. */
    public static final int TOP_PRIORITY    = 0;
    public static final int HIGH_PRIORITY   = 1;
    public static final int MEDIUM_PRIORITY = 2;
    public static final int LOW_PRIORITY    = 3;

    /** The number of different data priority levels. */
    public static final int MAX_PRIORITIES = 4;

    /* The various channel joining modes. */
    public static final int READONLY        = 0;
    public static final int WRITEONLY       = 1;
    public static final int READWRITE       = 2;

    /** The session that this channel belongs to. */
    protected Session session;

    /** Handle to client-side proxy. */
    public AbstractChannelProxy po;
 
    /** Handle to server-side object. */
    public AbstractChannelServer so;

    /** Indicates if data sent over this channel will be reliably delivered. */
    protected boolean reliable = false;
 
    /** Indicates whether data sent over this channel will be ordered. */
    protected boolean ordered = false;


/**
 * <A NAME="SD_CHANNEL"></A>
 * <EM>Channel</EM> the constructor for the Channel class.
 *
 * @param <EM>name</EM> the name of the channel being constructed.
 * The channel name cannot include '\u0001' or newline characters. 
 * @param <EM>session</EM> the session this channel belongs to.
 * @param <EM>reliable</EM> indicates whether the channel is reliable.
 * @param <EM>ordered</EM> indicates whether the channel is ordered.
 */

    public
    Channel(String name, Session session, boolean reliable, boolean ordered) {
        String className = null;

	if (Channel_Debug) {
            System.out.println("Channel: constructor:"  +
	    		       " name: "     + name     +
			       " session: "  + session  +
			       " reliable: " + reliable +
	    		       " ordered: "  + ordered);
        }

        this.name     = name;
        this.session  = session;
	this.reliable = reliable;
        this.ordered  = ordered;

        try {
	    className = "SharedData." + session.getConnectionType() +
		        ".ChannelProxy";
	    po = (AbstractChannelProxy) Class.forName(className).newInstance();
	    po.initProxy(name, session, this);

	    className = "SharedData." + session.getConnectionType() +
		        ".ChannelServer";
	    so = (AbstractChannelServer) Class.forName(className).newInstance();
	    so.initServer(name, session, this);
	} catch (Exception e) {
            System.out.println("Channel: constructor exception " + e);
            if (Channel_Debug) {
                e.printStackTrace();
            }
	}

        objectType = M_Channel;
        mpo = (AbstractSharedDataProxy) po;
    }


/**
 * <A NAME="SD_GETCONSUMER"></A>
 * <EM>getConsumer</EM> get the channel data consumer for this client on
 * this channel.
 *
 * @param <EM>client</EM> the client whose consumer will be returned.
 *
 * @return the data consumer for this client on this channel.
 */

    public ChannelConsumer
    getConsumer(Client client) {
        if (Channel_Debug) {
            System.out.println("Channel: getConsumer: " + client);
        }

        return(po.getConsumer(client));
    }


/**
 * <A NAME="SD_SETCONSUMER"></A>
 * <EM>setConsumer</EM> sets a new channel consumer for this client.
 * The client's channel consumer gets informed when there is data available
 * on a channel. If consumer is null, this has the effect of removing the
 * channel consumer for this client.
 *
 * @param <EM>client</EM> the client whose consumer will be set.
 * @param <EM>consumer</EM> the new channel consumer for this client.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *   NO_SUCH_CONSUMER  - consumer given does not exist.
 *   NO_SUCH_CHANNEL   - channel given does not exist.
 *   PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */
 
    public void
    setConsumer(Client client, ChannelConsumer consumer)
						throws SharedDataException {
        if (Channel_Debug) {
            System.out.println("Channel: setConsumer:" +
	    		       " client: "   + client  +
	    		       " consumer: " + consumer);
        }

	po.setConsumer(client, consumer);
    }


/**
 * <A NAME="SD_ISRELIABLE"></A>
 * <EM>isReliable</EM> test whether this is a reliable channel.
 *
 * @return true if this is a reliable channel; false if not.
 */
 
    public boolean
    isReliable() {
        if (Channel_Debug) {
            System.out.println("Channel: isReliable: " + reliable);
        }

	return(reliable);
    }


/**
 * <A NAME="SD_ISORDERED"></A>
 * <EM>isOrdered</EM> returns whether this is an ordered channel.
 *
 * @return true if this is an ordered channel; false if not.
 */

    public boolean
    isOrdered() {
        if (Channel_Debug) {
            System.out.println("Channel: isOrdered: " + ordered);
        }

        return(ordered);
    }


/**
 * <A NAME="SD_RECEIVE"></A>
 * <EM>receive</EM> is used to receive data from other clients joined to
 * this channel.
 *
 * It will block if there is no data to read. The <EM>dataAvailable</EM>
 * method can be used to check if there is data that can be received on
 * this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *   PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 *
 * @return the next Data object available on this channel.
 */

    public Data
    receive() throws SharedDataException {
        if (Channel_Debug) {
            System.out.println("Channel: receive.");
        }

        return(po.receive());
    }


/**
 * <A NAME="SD_DATAAVAILABLE"></A>
 * <EM>dataAvailable</EM> tests whether there is data available to read
 * on this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * The following exception types can occur:
 *   <currently none>
 *
 * @return true if data is available; false if not.
 */

    public boolean
    dataAvailable() throws SharedDataException {
        if (Channel_Debug) {
            System.out.println("Channel: dataAvailable.");
        }

        return(po.dataAvailable());
    }


/**
 * <A NAME="SD_SENDTOALL"></A>
 * <EM>sendToAll</EM> is used to send data to all clients joined to this
 * channel. If the sender is a consumer of this channel, then it will
 * receive the data.
 *
 * If multiple clients send data to the same channel, different receivers
 * may receive data from different senders in a different order. If uniform
 * sequencing is needed for some data, all senders should send that data
 * using <EM>uniformSendToAll</EM> described below.
 *
 * Data from each sender sent at the same priority on the same channel
 * arrives at a given receiver in the same order it was sent but may have
 * other sender data interleaved differently.
 *
 * @param <EM>sendingClient</EM> the client sending the data.
 * @param <EM>data</EM> the data being sent over this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *   NO_SUCH_CHANNEL   - channel given does not exist.
 *   PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */
 
    public void
    sendToAll(Client sendingClient, Data data) throws SharedDataException {
        if (Channel_Debug) {
            System.out.println("Channel: sendToAll:" +
                               " sending client: "       + sendingClient +
                               " receiving clients: all" +
			       " data:                 " + data);
        }
 
        po.send(sendingClient, D_All, data, false);
    }


/**
 * <A NAME="SD_SENDTOOTHERS"></A>
 * <EM>sendToOthers</EM> is used to send data to other clients joined to
 * this channel. The sender (irrespective of whether it's a consumer of this
 * channel) will not receive the data.
 *
 * If multiple clients send data to the same channel, different receivers
 * may receive data from different senders in a different order. If uniform
 * sequencing is needed for some data, all senders should send that data
 * using <EM>uniformSendToAll</EM> described below.
 *
 * Data from each sender sent at the same priority on the same channel
 * arrives at a given receiver in the same order it was sent but may have
 * other sender data interleaved differently.
 *
 * @param <EM>sendingClient</EM> the client sending the data.
 * @param <EM>data</EM> the data being sent over this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *   NO_SUCH_CHANNEL   - channel given does not exist.
 *   PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */
 
    public void
    sendToOthers(Client sendingClient, Data data) throws SharedDataException {
        if (Channel_Debug) {
            System.out.println("Channel: sendToOthers:" +
                               " sending client: "       + sendingClient +
                               " receiving clients: all" +
                               " data:                 " + data);
        }

        po.send(sendingClient, D_Others, data, false);
    }


/**
 * <A NAME="SD_SENDTOCLIENT"></A>
 * <EM>sendToClient</EM> is used to send data to a single client joined to
 * this channel.
 *
 * @param <EM>sendingClient</EM> the client sending the data.
 * @param <EM>receivingClient</EM> the name of the client receiving the data.
 * @param <EM>data</EM> the data being sent over this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_CHANNEL   - channel given does not exist.
 *      PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */
 
    public void
    sendToClient(Client sendingClient, String receivingClientName,
		 Data data) throws SharedDataException {
        if (Channel_Debug) {
            System.out.print("Channel: sendToClient:"   +
			     " sending client: "        + sendingClient +
			     " receiving client name: " + receivingClientName +
			     " data: "                  + data);
        }
 
        po.send(sendingClient, receivingClientName, data, false);
    }


/**
 * <A NAME="SD_UNIFORMSENDTOALL"></A>
 * <EM>uniformSendToAll</EM> is used to send data to all clients joined to
 * this channel in a uniformly sequenced manner, i.e. the data will be
 * received in the same sequence by all the members of the channel.
 *
 * Uniform Sequencing of data is guaranteed only for data of the same
 * priority on the same channel.
 *
 * If the sender is a consumer of the destination channel, it will receive
 * its own data, in the proper sequence. Therefore it is provided with
 * the same consistent data stream as other members of the channel.
 *
 * @param <EM>sendingClient</EM> the client sending the data.
 * @param <EM>data</EM> the data being sent over this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured. 
 * <PRE>
 * The following exception types can occur:
 *   NO_SUCH_CHANNEL   - channel given does not exist.
 *   PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */
 
    public void
    uniformSendToAll(Client sendingClient,
		     Data data) throws SharedDataException {
        if (Channel_Debug) {
            System.out.println("Channel: uniformSendToAll:" +
                               " sending client: "       + sendingClient +
                               " receiving clients: all" +
                               " data:                 " + data);
        }

        po.send(sendingClient, D_All, data, true);
    }


/**
 * <A NAME="SD_UNIFORMSENDTOOTHERS"></A>
 * <EM>uniformSendToOthers</EM> is used to send data to other clients
 * joined to this channel in a uniformly sequenced manner, i.e. the data
 * will be received in the same sequence by all the members of the channel.
 * The sender (irrespective of whether it's a consumer of this channel)
 * will not receive the data.
 *
 * Uniform Sequencing of data is guaranteed only for data of the same
 * priority on the same channel.
 *
 * The sender (irrespective of whether it's a consumer of this channel)
 * will not receive the data.
 *
 * @param <EM>sendingClient</EM> the client sending the data.
 * @param <EM>data</EM> the data being sent over this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *   NO_SUCH_CHANNEL   - channel given does not exist.
 *   PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */
 
    public void
    uniformSendToOthers(Client sendingClient,
                     Data data) throws SharedDataException {
        if (Channel_Debug) {
            System.out.println("Channel: uniformSendToOthers:" +
                               " sending client: "       + sendingClient +
                               " receiving clients: all" +
                               " data:                 " + data);
        }

        po.send(sendingClient, D_Others, data, true);
    }


/**
 * <A NAME="SD_JOIN"></A>
 * <EM>join</EM> is used by a client to join an appropriate channel whose
 * use is defined by the application. This is a prerequesite for sending
 * and receiving data sent to the channel.
 *
 * This method can throw a SharedDataException which can be of three types:
 *     - No such channel.
 *     - No such client.
 *     - Permission denied.
 *
 * @param <EM>session</EM> the session this channel belongs to.
 * @param <EM>channelName</EM> the name of the channel to join.
 * @param <EM>client</EM> identifies the client wishing to join this channel.
 *
 * @exception SharedDataException If a Shared Data exception has occured. 
 * <PRE>
 * The following exception types can occur:
 *   NO_SUCH_CHANNEL   - channel given does not exist.
 *   NO_SUCH_CLIENT    - client given does not exist.
 *   NAME_IN_USE       - this client name already exists.
 * </PRE>
 *
 * @return the Channel with this name in this session.
 */

    public static Channel
    join(Session session, String channelName, Client client)
                                             throws SharedDataException {
        return(join(session, channelName, client, READWRITE));
    }


/**
 * <A NAME="SD_JOINWITHMODE"></A>
 * <EM>join</EM> is used by a client to join an appropriate channel whose
 * use is defined by the application. This is a prerequesite for sending
 * and receiving data sent to the channel.
 *
 * This method can throw a SharedDataException which can be of three types:
 *     - No such channel.
 *     - No such client.
 *     - Permission denied.
 *
 * @param <EM>session</EM> the session this channel belongs to.
 * @param <EM>channelName</EM> the name of the channel to join.
 * @param <EM>client</EM> identifies the client wishing to join this channel.
 * @param <EM>mode</EM> the mode which can be one of, READONLY, WRITEONLY or
 * READWRITE.
 *
 * @exception SharedDataException If a Shared Data exception has occured. 
 * <PRE>
 * The following exception types can occur:
 *   NO_SUCH_CHANNEL   - channel given does not exist.
 *   NO_SUCH_CLIENT    - client given does not exist.
 *   NAME_IN_USE       - this client name already exists.
 *   PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 *
 * @return the Channel with this name in this session.
 */

    public synchronized static Channel
    join(Session session, String channelName, Client client, int mode)
                                            throws SharedDataException {
        return(session.po.joinChannel(session, channelName, client,
				      mode, true));
    }


/**
 * <A NAME="SD_LEAVE"></A>
 * <EM>leave</EM> removes a client from this channel. This client will no
 * longer be known to this channel. Observers of this channel will be sent
 * an indication when this happens.
 *
 * @param <EM>client</EM> the client in question.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *   NO_SUCH_CHANNEL - channel given does not exist.
 *   NO_SUCH_CLIENT  - client given does not exist.
 * </PRE>
 */

    public void
    leave(SharedData.Client client) throws SharedDataException {
        if (Channel_Debug) {
            System.out.println("Channel: leave: " + client);
        }

        po.leave(client);
    }
}
