
/*
 * @(#)Manageable.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data Manageable class.
 *
 * This is an abstract class that is the parent class for manageable
 * resources (Channels, Sessions and Tokens).
 *
 * @version     0.89 28 January 1997
 * @author      Rich Burridge
 */

public abstract
class Manageable extends SharedDataObject {

    /** The name of this object. */
    protected String name = null;

    /** The type of this object (Channel, Session or Token). */
    protected String objectType;

    /** Handle to manageable client-side proxy. */
    public AbstractSharedDataProxy mpo;


/**
 * <A NAME="SD_ADDOBSERVER"></A>
 * <EM>addObserver</EM> adds an observer to the list that will be informed
 * when any client undergoes any changes for this manageable object.
 *
 * @param <EM>observer</EM> the observer being added.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - for Session:addObserver
 *                      - session given does not exist.
 *      NO_SUCH_CHANNEL - for Channel:addObserver
 *                      - channel given does not exist.
 *      NO_SUCH_TOKEN   - for Token:addObserver
 *                      - token given does not exist.
 * </PRE>
 */

    public void
    addObserver(SharedData.SharedDataObserver observer)
					 throws SharedDataException {
        if (Manageable_Debug) {
            System.out.println("Manageable: addObserver: " + observer);
        }

        mpo.addObserver(observer, objectType);
    }


/**
 * <A NAME="SD_DELETEOBSERVER"></A>
 * <EM>deleteObserver</EM> deletes an observer from the list that will be
 * informed when any client undergoes any changes for this manageable
 * object.
 *
 * @param <EM>observer</EM> the observer being deleted.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION  - for Session:deleteObserver
 *                       - session given does not exist.
 *      NO_SUCH_CHANNEL  - for Channel:deleteObserver
 *                       - channel given does not exist.
 *      NO_SUCH_TOKEN    - for Token:deleteObserver
 *                       - token given does not exist.
 *      NO_SUCH_OBSERVER - observer given does not exist.
 * </PRE>
 */

    public void
    deleteObserver(SharedData.SharedDataObserver observer)
					 throws SharedDataException {
        if (Manageable_Debug) {
            System.out.println("Manageable: deleteObserver: " + observer);
        }
 
        mpo.deleteObserver(observer, objectType);
    }


/**
 * <A NAME="SD_EXPEL"></A>
 * <EM>expel</EM> expel a client from a session.
 *
 * This method should only be called by the manager for this object.
 * An indication is delivered to each observer of this manageable object,
 * for each client expelled.
 *
 * @param <EM>clients</EM> the list of clients to be expelled from this
 * object.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION   - for Session:expel
 *                        - session given does not exist.
 *      NO_SUCH_CHANNEL   - for Channel:expel
 *                        - channel given does not exist.
 *      NO_SUCH_TOKEN     - for Token:expel
 *                        - token given does not exist.
 *      NO_SUCH_CLIENT    - client given does not exist.
 *      PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */

    public void
    expel(SharedData.Client[] clients) throws SharedDataException {
        if (Manageable_Debug) {
            System.out.println("Manageable: expel.");
        }

        mpo.expel(clients, objectType);
    }


/**
 * <A NAME="SD_GETNAME"></A>
 * <EM>getName</EM> get the name of this object.
 *
 * @return the name of this object.
 * The object name cannot include '\u0001' or newline characters. 
 */

    public String
    getName() {
        if (Manageable_Debug) {
            System.out.println("Manageable: getName: " + name);
        }
        return(name);
    }


/**
 * <A NAME="SD_INVITE"></A>
 * <EM>invite</EM> invite a client to join a manageable object.
 *
 * This method should only be called by the manager for this object.
 * An indication is delivered to each observer of this manageable object,
 * for each client invited.
 *
 * @param <EM>clients</EM> the list of clients to be invited to join this
 * object.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION   - for Session:invite
 *                        - session given does not exist.
 *      NO_SUCH_CHANNEL   - for Channel:invite
 *                        - channel given does not exist.
 *      NO_SUCH_TOKEN     - for Token:invite
 *                        - token given does not exist.
 *      NO_SUCH_CLIENT    - client given does not exist.
 *      PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 */

    public void
    invite(SharedData.Client[] clients) throws SharedDataException {
        if (Manageable_Debug) {
            System.out.println("Manageable: invite.");
        }
 
        mpo.invite(clients, objectType);
    }


/**
 * <A NAME="SD_LEAVE"></A>
 * <EM>leave</EM> removes a client from this manageable object. This client
 * will no longer be known to this object. Observers of this object will be
 * sent an indication when this happens.
 *
 * @param <EM>client</EM> the client in question.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - for Session:leave
 *                      - session given does not exist.
 *      NO_SUCH_TOKEN   - for Token:leave
 *                      - token given does not exist.
 *      NO_SUCH_CLIENT  - client given does not exist.
 * </PRE>
 */

    public void
    leave(SharedData.Client client) throws SharedDataException {
        if (Manageable_Debug) {
            System.out.println("Manageable: leave: " + client);
        }

        mpo.leave(client, objectType);
    }


/**
 * <A NAME="SD_LISTCLIENTNAMES"></A>
 * <EM>listClientNames</EM> list the names of the clients who are joined
 * to this manageable object.
 *
 * @return an array of names of clients currently joined to this object.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_SESSION - for Session:listClientNames
 *                      - session given does not exist.
 *      NO_SUCH_CHANNEL - for Channel:listClientNames
 *                      - channel given does not exist.
 *      NO_SUCH_TOKEN   - for Token:listClientNames
 *                      - token given does not exist.
 * </PRE>
 */
 
    public String[]
    listClientNames() throws SharedDataException {
        if (Manageable_Debug) {
            System.out.println("Manageable: listClientNames.");
        }
 
        return(mpo.listClientNames(objectType));
    }
}
