
/*
 * @(#)Token.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data Token class.
 *
 * @version     0.94 3 April 1997
 * @author      Rich Burridge
 */

public class
Token extends Manageable {

    /** To signify a manageable object of type Token. */
    public static final String M_Token   = "Token";

    /* The possible states a token can be in. */
    public static final int NOT_IN_USE      = 100;
    public static final int GRABBED         = 101;
    public static final int INHIBITED       = 102;
    public static final int GIVING          = 103;
    public static final int ALREADY_GRABBED = 104;

    /** The time to wait (in seconds) for token give() operations. */
    public static final int MAX_GIVE_TIME = 15;

    /** The session that this token belongs to. */
    private Session session;

    /** The token manager associated with this token (if any). */
    protected TokenManager tokenManager = null;

    /** Handle to client-side proxy. */
    public AbstractTokenProxy po;
 
    /** Handle to server-side object. */
    public AbstractTokenServer so;

    /** Indicates whether the token is being grabbed exclusively. */
    public boolean isExclusive = false;

    /** The status of this token. */
    public Integer status;


/**
 * <A NAME="SD_TOKEN"></A>
 * <EM>Token</EM> the constructor for the Token class.
 *
 * @param <EM>name</EM> the name of the token being constructed.
 * The token name cannot include '\u0001' or newline characters. 
 * @param <EM>session</EM> the session this token belongs to.
 */

    public
    Token(String name, Session session) {
        String className = null;

	if (Token_Debug) {
            System.out.println("Token: constructor:"  +
                               " name: "    + name    +
                               " session: " + session);
        }

	this.name    = name;
        this.session = session;
	this.status  = new Integer(NOT_IN_USE);

	try {
            className = "SharedData." + session.getConnectionType() +
                        ".TokenProxy";
            po = (AbstractTokenProxy) Class.forName(className).newInstance();
            po.initProxy(name, session, this);

            className = "SharedData." + session.getConnectionType() +
                        ".TokenServer";
            so = (AbstractTokenServer) Class.forName(className).newInstance();
            so.initServer(name, session, this);
	} catch (Exception e) {
            System.out.println("Token_Debug: constructor exception " + e);
            if (Token_Debug) {
                e.printStackTrace();
            }
	}

	objectType = M_Token;
	mpo = (AbstractSharedDataProxy) po;
    }


/**
 * <A NAME="SD_JOIN"></A>
 * <EM>join</EM> is used by a client to join an appropriate token whose
 * use is defined by the application. This is a prerequesite before any of
 * the various token operations can be done.
 *
 * This method with throw a SharedDataException which can be of three types:
 *     - No such token.
 *     - No such client.
 *     - Permission denied.
 *
 * @param <EM>session</EM> the session this token belongs to.
 * @param <EM>tokenName</EM> the name of the token to join.
 * @param <EM>client</EM> identifies the client wishing to join this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured. 
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN     - token given does not exist.
 *      NO_SUCH_CLIENT    - client given does not exist.
 *      NAME_IN_USE       - this client name already exists.
 *      PERMISSION_DENIED - you do not have permission for this operation.
 * </PRE>
 *
 * @return the Token with this name in this session.
 */

    public synchronized static Token
    join(Session session, String tokenName, Client client)
                                            throws SharedDataException {
        return(session.po.joinToken(session, tokenName, client));
    }


/**
 * <A NAME="SD_GRAB"></A>
 * <EM>grab</EM> is used to take exclusive (grab) or non-exclusive (inhibit)
 * control of a specific token.
 *
 * With an exclusive grab, the grab will succeed if the requestor is the sole
 * inhibitor of the token.
 * 
 * The non-exclusive grab is used to take non-exclusive control of a specific
 * token. It is used to prevent someone else from exclusively grabbing the
 * token. Several clients could inhibit a token at the same time. This
 * operation will succeed if the requestor has grabbed the token. The result
 * will be that the token is no longer grabbed exclusively and is inhibited
 * instead. Therefore it may be inhibited by other clients too.
 *
 * @param <EM>client</EM> the client grabbing/inhibiting this token.
 * @param <EM>exclusive</EM> indicates whether the grab should be exclusive.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN  - token given does not exist.
 *      NO_SUCH_CLIENT - client given does not exist.
 * </PRE>
 *
 * @return the status of the token.
 */

    public int
    grab(Client client, boolean exclusive) throws SharedDataException {
        if (Token_Debug) {
            System.out.println("Token: grab:"  +
                                " client: "    + client +
                                " exclusive: " + exclusive);
        }

	return(po.grab(client, exclusive));
    }


/**
 * <A NAME="SD_GIVE"></A>
 * <EM>give</EM> is used by an application client to surrender a token to
 * another application client.
 *
 * <EM>give</EM> will fail if the requestor has not grabbed the specified
 * token within a certain period of time.
 *
 * A <EM>GIVE</EM> TokenEvent is delivered to the Token Observer of the
 * client that this client would like to give the token to.
 *
 * A token being passed between two clients and whose possession is not yet
 * resolved will appear to any client requesting <EM>test</EM> to be grabbed
 * by some other client and not held by the requestor. <EM>grab</EM> will fail
 * during this interval, even if requested by one of the two clients involved.
 * <EM>release</EM> requested by the given will succeed, with the result that
 * the token is released if the offer to the recipient is ultimately rejected.
 * <EM>release</EM> by the recipient will have no effect, just like the
 * attempted release of any other token that the requester does not yet possess.
 * During the interval that a token is being passed any <EM>please</EM>
 * indications that are generated will be delivered to both clients involved.
 *
 * It is possible for the ownership of a passed token to be resolved
 * before a <EM>take</EM> or <EM>given</EM> is issued. Namely, deletion
 * of either the giver or recipient will leave the survivor in sole
 * possession as grabber of the token. At this point, the special effects
 * explained will cease to apply.
 *
 * @param <EM>client</EM> the client giving this token.
 * @param <EM>receivingClientName</EM> the name of the client to receive
 * the token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN  - token given does not exist.
 *      NO_SUCH_CLIENT - client given does not exist.
 * </PRE>
 *
 * @return the status of the token.
 */

    public int
    give(Client client, String receivingClientName) throws SharedDataException {
        if (Token_Debug) {
            System.out.println("Token: give:" +
		       " client: " + client +
		       " receiving client name: " + receivingClientName);
        }

        return(po.give(client, receivingClientName));
    }


/**
 * <A NAME="SD_PLEASE"></A>
 * <EM>please</EM> is used by a client to request a token from the
 * current possessor(s) of the token. A token may be inhibited
 * by several clients, or it may be grabbed by one client. In any case, a
 * <EM>PLEASE</EM> TokenEvent is delivered to all the Token Observers
 * observing this token, who currently possess the token.
 *
 * @param <EM>client</EM> the client requesting this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN - token given does not exist.
 * </PRE>
 *
 * @return the status of the token.
 */

    public int
    please(Client client) throws SharedDataException {
        if (Token_Debug) {
            System.out.println("Token: please:" +
			       " client: " + client);
        }

        return(po.please(client));
    }


/**
 * <A NAME="SD_RELEASE"></A>
 * <EM>release</EM> is used to free up a previously grabbed/inhibited token.
 *
 * @param <EM>client</EM> the client releasing this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN       - token given does not exist.
 *      NO_SUCH_CLIENT      - client given does not exist.
 *      CLIENT_NOT_GRABBING - this client is not grabbing this token.
 *      CLIENT_NOT_RELEASED - the token was not released successfully.
 * </PRE>
 *
 * @return the status of the token.
 */

    public int
    release(Client client) throws SharedDataException { 
        if (Token_Debug) {
            System.out.println("Token: release:"  +
                                " client: " + client);
        }

        return(po.release(client));
    }


/**
 * <A NAME="SD_TEST"></A>
 * <EM>test</EM> is used to check whether a token is available.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN - token given does not exist.
 * </PRE>
 *
 * @return one of the following:
 * <PRE>
 * NOT_IN_USE : not in use
 * GRABBED    : grabbed
 * INHIBITED  : inhibited
 * GIVING     : giving
 * </PRE>
 */

    public int
    test() throws SharedDataException {
        if (Token_Debug) {
            System.out.println("Token: test.");
        }

        return(po.test());
    }
}
