
/*
 * @(#)ChannelServer.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.*;

/**
 * Shared Data Channel Server-side class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

public class
ChannelServer extends SharedDataServer implements AbstractChannelServer {

    /** The server-side session associated with this channel server. */
    private Session session;

    /** The server-side channel associated with this channel server. */
    private Channel channel;

    /** Clients currently joined to this channel and their data direction. */
    protected Hashtable clientDataDirections = null;

    /** The consumers for the various clients on this channel (one for
     *  each priority level.
     */
    protected Hashtable clientConsumers[];

    /** The consumers and their associated channel threads (one for
     *  each priority level).
     */
    protected Hashtable consumerThreads[];


/**
 * <A NAME="SD_INITSERVER"></A>
 * <EM>initServer</EM> initialise the "server-side" for this Channel.
 *
 * @param <EM>name</EM> the name of the server Channel being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>session</EM> the server-side session the Channel belongs to.
 * @param <EM>object</EM> the server-side Channel.
 */

    public void
    initServer(String name, Session session, Object object) {
        if (ChannelServer_Debug) {
            System.out.println("ChannelServer: initServer:" +
            		       "name: "     + name          +
	    		       " session: " + session       +
	    		       " channel: " + channel);
        }

        this.name       = name;
	this.session    = session;
	this.channel    = (Channel) object;

        clients              = new Hashtable();
        clientDataDirections = new Hashtable();
        consumerThreads      = new Hashtable[Channel.MAX_PRIORITIES];
	clientConsumers      = new Hashtable[Channel.MAX_PRIORITIES];
	for (int priority = 0; priority < Channel.MAX_PRIORITIES; priority++) {
            consumerThreads[priority] = new Hashtable();
            clientConsumers[priority] = new Hashtable();
	}

        super.initServer(name, session, object);
    }


    public Object
    getServer() {
        if (ChannelServer_Debug) {
            System.out.println("ChannelServer: getServer.");
        }
 
        return(this);
    }


/**
 * <A NAME="SD_JOIN"></A>
 * <EM>join</EM> join a client to a channel. A message is returned to the
 * calling proxy, to indicate the success or failure of this operation.
 * If the client hasn't already joined this session, then the return value
 * is non-zero, indicating an exception will be thrown on the other side.
 * If it's a valid client, then all the observers of this channel are
 * informed that a new client has joined.
 * If successful, part of the return message to the calling proxy indicates
 * whether the channel the client has joined is reliable or not.
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>session</EM> the session this channel belongs to.
 * @param <EM>thread</EM> the thread for this calling proxy.
 * @param <EM>clientName</EM> the name of the client joining the channel.
 * @param <EM>mode</EM> the mode of joining (read only, write only or read
 * and write).
 *
 * @return the message to send back to the proxy.
 */

    String
    join(int id, Session session, SocketThread thread, 
	 String clientName, int mode) {
        int    retval = 0;
        Client client = new ClientImpl(clientName);

        if (ChannelServer_Debug) {
            System.out.println("ChannelServer: join:"  +
				" id: "          + id +
				" session: "     + session +
				" thread: "      + thread +
				" client name: " + clientName +
				" mode: "        + mode);
        }

        if (((SessionServer) session.so).validClient(client) == false) {
            retval = SharedDataException.NO_SUCH_CLIENT;
        } else {
            if (retval == 0) {
                synchronized (clients) {
		    if (clients.containsKey(clientName)) {
                	retval = SharedDataException.NAME_IN_USE;
            	    } else {
			clients.put(clientName, client);
		    }
		}

                if (retval == 0) {
		    synchronized (clientDataDirections) {
		        clientDataDirections.put(client, new Integer(mode));
		    }

                    informObservers(session.getName(), clientName,
				    SharedDataEvent.JOIN, Channel.M_Channel);
	            addClientThreadConnection(clientName, thread);
		}
            }
	}

        return(addMessageHeader(session.getName(), id,
				Channel.M_Channel + T_Sep + T_Join + T_Sep +
				retval + T_Sep + channel.isReliable()));
    }


/**
 * <A NAME="SD_LEAVE"></A>
 * <EM>leave</EM>
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>session</EM> the server-side session this channel belongs to.
 * @param <EM>clientName</EM> the name of the client leaving this Channel.
 * @param <EM>objectType</EM> the server-side Channel.
 *
 * @return the message to send back to the proxy.
 */

    protected String
    leave(int id, Session session, String clientName, String objectType) {
	Client client = getClientByName(clientName);
	String reply  = super.leave(id, session, clientName, objectType);

        if (ChannelServer_Debug) {
            System.out.println("ChannelServer: leave:" +
                                " id: "          + id +
				" session: "     + session   +
                                " client name: " + clientName);
        }

	if (client != null) {
	    if (clientDataDirections != null) {
		synchronized (clientDataDirections) {
                    clientDataDirections.remove(client);
		}
            }   

            for (int i = 0; i < Channel.MAX_PRIORITIES; i++) {
                if (consumerThreads[i] != null && clientConsumers[i] != null) {
		    ChannelConsumer consumer = null;

		    synchronized (clientConsumers[i]) {
		        consumer = (ChannelConsumer) 
				    clientConsumers[i].get(client);
		        clientConsumers[i].remove(client);
		    }
		    synchronized (consumerThreads[i]) {
		        consumerThreads[i].remove(consumer);
                    }
                }
	    }
	}

	return(reply);
    }


/**
 * <A NAME="SD_SEND"></A>
 * <EM>send</EM> is used to send a data message to the appropriate consumers
 * of the channel. This can be a single client, all clients or all other
 * clients except the sender.
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>session</EM> the session containing the channel over which the
 * data was sent.
 * @param <EM>channelName</EM> the name of the channel over which the data
 * was sent.
 * @param <EM>senderName</EM> the name of the client sending the data.
 * @param <EM>receiverName</EM> the name of the client receiving the data
 * (or the special tokens D_All or D_Others).
 * @param <EM>priority</EM> the priority level the data was sent at.
 * @param <EM>length</EM> the length of the data (in bytes).
 * @param <EM>uniform</EM> set true if this is a uniform send.
 * @param <EM>data</EM> the data being sent over this channel.
 *
 * @return the message to send back to the proxy.
 */

    String
    send(int id, Session session, String channelName, String senderName,
	 String receiverName, int priority, int length, boolean uniform,
	 byte data[]) {
        int    retval  = 0;
	Client sender  = null;
	String message = addMessageHeader(session.getName(), 0,
		 	   Channel.M_Channel + T_Sep + T_DataReceived + T_Sep +
		 	   channelName + T_Sep + senderName + T_Sep);

        if (ChannelServer_Debug) {
            System.out.println("ChannelServer: send: " + 
                                " id: "            + id +
                                " session: "       + session +
                                " channel name: "  + channelName +
                                " sender name: "   + senderName +
                                " receiver name: " + receiverName +
                                " priority: "      + priority +
                                " length: "        + length +
                                " uniform: "       + uniform);
        }

	sender = getClientByName(senderName);
	synchronized (clientDataDirections) {
	    if (((Integer) clientDataDirections.get(sender)).intValue() ==
						     Channel.READONLY) {
	        retval = SharedDataException.PERMISSION_DENIED;
	    }
	}

	if (retval == 0) {
	    if (receiverName.equals(Channel.D_All) == true ||
		receiverName.equals(Channel.D_Others) == true) {
		if (consumerThreads[priority] != null) {
		    synchronized (consumerThreads[priority]) {
		        Enumeration e, k;

			for (e = consumerThreads[priority].elements(),
			     k = consumerThreads[priority].keys();
				     e.hasMoreElements();) {
			    SocketThread thread =
					 (SocketThread) e.nextElement();
			    ChannelConsumerImpl consumer =
					 (ChannelConsumerImpl) k.nextElement();
			    Client client = consumer.getClient();
			    String dataMessage = message + client.getName() +
				 T_Sep + priority + T_Sep + length + T_Sep;

			    if (receiverName.equals(Channel.D_Others) == true &&
				client.getName().equals(senderName) == true) {
				continue;
			    }

			    thread.writeData(null, dataMessage, data, length);
			}
		    }
		}
	    } else {
	        SocketThread thread = getThreadForClient(receiverName);

	        if (thread != null) {
                    String dataMessage = message + receiverName + T_Sep +
					 priority + T_Sep + length + T_Sep;

		    thread.writeData(null, dataMessage, data, length);
	        } else {
                    System.out.println("ChannelServer: serverSend:" +
                               " no thread for client name: " + receiverName);
	        }
	    }
	}
	return(addMessageHeader(session.getName(), id,
			Channel.M_Channel + T_Sep + T_Send + T_Sep + retval));
    }


/**
 * <A NAME="SD_SETCONSUMER"></A>
 * <EM>setConsumer</EM>
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>thread</EM> the thread for this calling proxy.
 * @param <EM>sessionName</EM> the name of the session.
 * @param <EM>clientName</EM> the name of the client whose consumer will be set.
 * @param <EM>consumerName</EM> the name of the consumer
 *
 * @return the message to send back to the proxy.
 */

    String
    setConsumer(int id, SocketThread thread, String sessionName,
		String clientName, String consumerName, int priority) {
        Client              client   = getClientByName(clientName);
        ChannelConsumerImpl consumer = null;
        int                 retval   = 0;
 
        if (ChannelServer_Debug) {
            System.out.println("ChannelServer: setConsumer" +
				" id: "            + id +
				" session name: "  + sessionName +
				" client name: "   + clientName + 
				" consumer name: " + consumerName +
				" priority: "      + priority);
        }
 
        if (clientDataDirections == null) {
	    synchronized (clientDataDirections) {
		if (((Integer) clientDataDirections.get(client)).intValue() ==
						     Channel.WRITEONLY) {
                    retval = SharedDataException.PERMISSION_DENIED;
		}
	    }
	}

	if (retval == 0) {
            synchronized (clientConsumers[priority]) {
		if (consumerName.equals(T_Null)) {
		    consumer = (ChannelConsumerImpl)
				clientConsumers[priority].get(client);

		    if (clientConsumers[priority].remove(client) == null) {
			retval = SharedDataException.NO_SUCH_CONSUMER;
		    }
		} else {
		    consumer = new ChannelConsumerImpl(consumerName, client);
		    clientConsumers[priority].put(client, consumer);
		}
	    }

            synchronized (consumerThreads[priority]) {
		if (consumerName.equals(T_Null) && consumer != null) {
		    consumerThreads[priority].remove(consumer);
		} else {
		    consumerThreads[priority].put(consumer, thread);
		}
	    }
	}
	return(addMessageHeader(sessionName, id, Channel.M_Channel +
				T_Sep + T_SetConsumer + T_Sep + retval));
    }
}
