
/*
 * @(#)NamingProxy.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.io.IOException;
import java.util.*;

/**
 * Shared Data Naming proxy class.
 *
 * This class provides simple URL based naming for Shared Data objects.
 *
 * Based on the RMI Naming class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

public class
NamingProxy extends socketSharedDataObject implements AbstractNamingProxy {

    /** The sessions currently bound. */
    private Hashtable sessions = null;

    /** The clients currently bound. */
    private Hashtable clients = null;

    /** The Naming class instances for the various hosts. */
    private static Hashtable connections = null;

    /** The client-side thread for contacting the registry. */
    private SocketThread proxyThread = null;


    public void
    initProxy(String host, int port) {
     System.out.println("Inside initProxy method");
        Thread thread;

	if (NamingProxy_Debug) {
            System.out.println("NamingProxy: initProxy:" +
			       " host: " + host +
			       " port: " + port);
        }

        sessions = new Hashtable();
        System.out.println("Hashtable for sessions created");
        clients  = new Hashtable();

	proxyThread = new TCPSocketThread(host, port);
	thread = new Thread(proxyThread);
      System.out.println("Thread spawned at "+host+" in port "+port);
	thread.start();
       System.out.println("Spawned thread started..");
    }


/**
 * <A NAME="SD_GETPROXY"></A>
 * <EM>getProxy</EM>
 *
 * @return a handle to the proxy for this object. 
 */

    public Object
    getProxy() {
	return(this);
    }


/**
 * <A NAME="SD_BIND"></A>
 * <EM>bind</EM> binds the name to the specified Shared Data object.
 *
 * The name should be of the form:
 *     coll://<machine>[:<port>]/<objecttype>/<objectname>
 *
 * where valid object types are "Session" and "Client".
 *
 * So for example:
 *     bind("coll://stard:3355/Session/chat.session", chatSession);
 *
 *     bind("coll://jsda:4386/Client/fred.client", fredClient);
 *
 * @param <EM>name</EM> the name associated with this object.
 * @param <EM>object</EM> the object to bind.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * The following exception types can occur:
 * <PRE>
 *      INVALID_URL    - url string given is invalid.
 *      ALREADY_BOUND  - this url string is already bound to an object.
 * </PRE>
 */

    public void
    bind(Naming naming, URLString urlString,
	 String name, Object object) throws SharedDataException {
        if (NamingProxy_Debug) {
            System.out.println("NamingProxy: bind:"         +
				" naming: "     + naming    +
				" URL string: " + urlString +
				" name: "       + name      +
				" object: "     + object);
        }
	_bind(naming, urlString, name, object, 
	      T_Bind + T_Sep + proxyThread.uniqueName(this, name));
    }


/**
 * <A NAME="SD_REBIND"></A>
 * <EM>rebind</EM> rebinds the name to a new object, replacing any existing
 * binding.
 *
 * @param <EM>name</EM> the name associated with this new object.
 * @param <EM>object</EM> the object to rebind.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * The following exception types can occur:
 * <PRE>
 *      INVALID_URL - url string given is invalid.
 * </PRE>
 */

    public void
    rebind(Naming naming, URLString urlString,
	   String name, Object object) throws SharedDataException {
        if (NamingProxy_Debug) {
            System.out.println("NamingProxy: rebind:" +
				" naming: "     + naming   +
				" URL string: " + urlString +
				" name: "       + name     +
				" object: "     + object);
        }
        _bind(naming, urlString, name, object, 
	      T_Rebind + T_Sep + proxyThread.uniqueName(this, name));
    }


/**
 * <A NAME="SD_UNBIND"></A>
 * <EM>unbind</EM> unbinds the object associated with this name.
 *
 * @param <EM>name</EM> the name of the object to unbind.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      INVALID_URL - url string given is invalid.
 *      NOT_BOUND   - no object bound to this url string.
 * </PRE>
 */
 
    public void
    unbind(Naming naming, URLString urlString,
	   String name, Object object) throws SharedDataException {
	NamingProxy np     = (NamingProxy) naming.po.getProxy();
	String waitMessage = T_Unbind + T_Sep + proxyThread.uniqueName(this, name);
        String message     = waitMessage + T_Sep + name;
 
        if (NamingProxy_Debug) {
            System.out.println("NamingProxy: unbind: " + message);
        }
 
        sendMessage(np.proxyThread, waitMessage, message, null, 0);
    }


/**
 * <A NAME="SD_LOOKUP"></A>
 * <EM>lookup</EM> returns the Shared Data object for the given name.
 *
 * The name should be of the form:
 *     coll://<machine>[:<port>]/<connectType>/<objectType>/<objectName>
 *
 * where valid object types are "Session" and "Client".
 *
 * So for example:
 *   Session session = lookup("coll://stard:3355/socket/Session/chat.session");
 *
 *   Client  client  = lookup("coll://jsda:4386/socket/Client/fred.client");
 *
 * @param <EM>name</EM> the name of the object to find.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      INVALID_URL - url string given is invalid.
 *      NOT_BOUND   - no object bound to this url string.
 * </PRE>
 *
 * @return the resolved object.
 */

    public Object
    lookup(Naming naming, URLString urlString,
	   String name) throws SharedDataException {
        System.out.println("Inside naming.po.lookup..");
       System.out.println("Before naming.po.getProxy()");
        NamingProxy np     = (NamingProxy) naming.po.getProxy();
        System.out.println("After naming.po.getProxy()");
	String waitMessage = T_Lookup + T_Sep + proxyThread.uniqueName(this, name);
        String message     = waitMessage + T_Sep + name;
        String connectType = urlString.getConnectionType();
	String objectType  = urlString.getObjectType();
	String objectName  = urlString.getObjectName();
        String host        = urlString.getHost();
System.out.println("Hostn from urlString is "+host);
	int    port        = urlString.getPort();
System.out.println("Port from urlString is "+port);
 
        if (NamingProxy_Debug) {
            System.out.println("NamingProxy: lookup: " + message);
        }

	sendMessage(np.proxyThread, waitMessage, message, null, 0);
    System.out.println("After sendMessage method...");

	try {
	    if (objectType.equals(Session.M_Session) == true) {
	        socketSession session = null;

	        synchronized (sessions) {
		    session = (socketSession) sessions.get(objectName);
		    if (session == null) {
		        String className = "SharedData." + connectType +
				           "." + connectType + "Session";

			session = (socketSession)
			        Class.forName(className).newInstance();

		        sessions.put(objectName, session);
                        session._createProxy(objectName, connectType,
					     host, port);
		    }
	        }
	        return((Object) session);
	    } else if (objectType.equals(Client.M_Client) == true) {
	        socketClient client = null;

		synchronized (clients) {
		    client = (socketClient) clients.get(objectName);
		    if (client == null) {
                        String className = "SharedData." + connectType +
                                           "." + connectType + "Client";
    
		        client = (socketClient)
			        Class.forName(className).newInstance();
    
		        clients.put(objectName, client);
                        client._createProxy(objectName, connectType, 
					    host, port);
		    } 
	        }
  	        return((Object) client);
	    }
        } catch (Exception e) { 
            System.out.println("NamingProxy: lookup: exception: " + e);
            if (NamingProxy_Debug) {
                e.printStackTrace();
            }
        }

	return(null);
    }


/**
 * <A NAME="SD_LIST"></A>
 * <EM>list</EM> lists all the names of the known bound Shared Data objects.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      <currently none>
 * </PRE>
 *
 * @return an array of Strings of the names of all the known bound Shared
 * Data objects.
 */

    public String[]
    list(Naming naming) throws SharedDataException {
        NamingProxy     np    = (NamingProxy) naming.po.getProxy();
        String[]        names = null;
        StringTokenizer tok   = null;
	String waitMessage = T_List + T_Sep + proxyThread.uniqueName(this, "LIST");
        String message     = waitMessage;
        Vector objectNames = new Vector();
        int    n           = 0;

        if (NamingProxy_Debug) {
            System.out.println("NamingProxy: list: " + message);
        }

        tok = sendMessage(np.proxyThread, waitMessage, message, null, 0);

        if ((n = Integer.parseInt(tok.nextToken())) != 0) {
            for (int i = 0; i < n; i++) {
                objectNames.addElement(tok.nextToken());
            }
        }

        names = new String[n];
        objectNames.copyInto(names);
        return(names);
    }


/**
 * <A NAME="SD__BIND"></A>
 * <EM>_bind</EM>
 *
 * @param <EM>name</EM>
 * @param <EM>object</EM>
 * @param <EM>waitMessage</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */

    private void
    _bind(Naming naming, URLString urlString, String name,
	  Object object, String waitMessage) throws SharedDataException {
        NamingProxy np          = (NamingProxy) naming.po.getProxy();
	String      message     = waitMessage + T_Sep + name;

        if (NamingProxy_Debug) {
            System.out.println("NamingProxy: _bind: " + message);
        }

        sendMessage(np.proxyThread, waitMessage, message, null, 0);

        try {
	    int    port        = urlString.getPort();
            String connectType = urlString.getConnectionType();
	    String objectType  = urlString.getObjectType();
            String objectName  = urlString.getObjectName();
	    String className = "SharedData." + connectType + "." +
		                connectType + objectType;

	    if (object.getClass() == Class.forName(className)) {
	        if (objectType.equals(Session.M_Session)) {
                    ((socketSession) object)._createServer(objectName,
                                                   connectType, name, port);
                } else if (objectType.equals(Client.M_Client)) {
                    ((socketClient) object)._createServer(objectName,       
                                                   connectType, name, port);    
                } else {
                    throw new SharedDataException(
                                        SharedDataException.INVALID_URL);
                }
	    }
        } catch (Exception e) {
            System.out.println("NamingProxy: _bind: exception: " + e);
            if (NamingProxy_Debug) {
                e.printStackTrace();
            }
        }
    }
}
