
/*
 * @(#)SocketThread.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.Socket;
import java.io.*;
import java.util.Vector;

/**
 * Shared Data Socket thread class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

class
SocketThread extends socketSharedDataObject implements Runnable {

    /** Maximum number of messages allowed in the message queue. */
    public static final int MAXQUEUE = 25;

    /** Message queue (from server). */
    protected Vector messages = null;

    /** The host of the server machine. */
    protected String host;

    /** The port number to communicate on. */
    protected int port;

    /** Am I waiting for a certain command? */
    protected boolean isWaiting = false;

    /** Command the client is waiting to complete (if any). */
    protected String waitCommand = null;


/**
 * <A NAME="SD_MESSAGECONTAINSDATA"></A>
 * <EM>messageContainsData</EM> checks if this is one of the message types
 * that contains data. This methos should be over-ridden by subclasses if
 * they explicitly handle messages that can contain data.
 *
 * @param <EM>type</EM>   - the type of this message  
 * @param <EM>action</EM> - the action for this message.
 */
 
    private boolean
    messageContainsData(String type, String action) {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: messageContainsData: " +
			       " type: "   + type +
			       " action: " + action);
        }

        return(false);
    }


/**
 * <A NAME="SD_READBYTES
 * <EM>readBytes</EM>
 *
 * @param <EM>length</EM>
 *
 * @return
 */

    synchronized byte[]
    readBytes(int length) {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: readBytes: " + length);
        }

        System.out.println("SocketThread: readBytes: " +
                           "Should be over-ridden by it's subclass.");

	return(null);
    }


/**
 * <A NAME="SD_WRITEDATA"></A>
 * <EM>writeData</EM> write a message (with data) over the socket.
 *
 * @param <EM>waitCommand</EM> the reply to wait for (if any).
 * @param <EM>message</EM> the message to write to the socket.
 * @param <EM>dataBytes</EM> the data.
 * @param <EM>length</EM> the length (in bytes) of the data.
 *
 * @return the reply (if we were waiting for it), otherwise null.
 */

    String
    writeData(String waitCommand, String message,
	      byte dataBytes[], int length) {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: writeData.");
        }

        return(_write(waitCommand, message, dataBytes, length));
    }


/**
 * <A NAME="SD_WRITELINE"></A>
 * <EM>writeLine</EM> write a message over the socket.
 *
 * @param <EM>waitCommand</EM> the reply to wait for (if any).
 * @param <EM>message</EM> the message to write to the socket.
 *
 * @return the reply (if we were waiting for it), otherwise null.
 */

    String
    writeLine(String waitCommand, String message) {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: writeLine.");
        }

	return(_write(waitCommand, message, null, 0));
    }


/**
 * <A NAME="SD__WRITE"></A>
 * <EM>_write</EM> write a message (possibly with data) over the socket.
 *
 * @param <EM>waitCommand</EM> the reply to wait for (if any).
 * @param <EM>message</EM> the message to write to the socket.
 * @param <EM>dataBytes</EM> the data (or null if there isn't any).
 * @param <EM>length</EM> the length (in bytes) of the data.
 *
 * @return the reply (if we were waiting for it), otherwise null.
 */

    synchronized String
    _write(String waitCommand, String message, byte dataBytes[], int length) {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: _write.");
        }

        System.out.println("SocketThread: _write: " +
                           "Should be over-ridden by it's subclass.");
	return(null);
    }


/**
 * <A NAME="SD_GETDATA"></A>
 * <EM>getData</EM>
 *
 * @exception InterruptedException
 *
 * @return
 */

    byte[]
    getData() throws InterruptedException {
        String message = null;
        byte   data[]  = null;
	int    length  = 0;

        if (SocketThread_Debug) {
            System.out.println("SocketThread: getData.");
        }

        message = getMessage();
	length = message.length();
	data = new byte[length];
	message.getBytes(0, length, data, 0);
        return(data);
    }


/**
 * <A NAME="SD_GETID"></A>
 * <EM>getId</EM>
 *
 * @return a unique id for this thread.
 */

    int
    getId() {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: getId.");
        }

        return(this.hashCode());
    }


/**
 * <A NAME="SD_GETMESSAGE"></A>
 * <EM>getMessage</EM>
 *
 * @exception InterruptedException
 *
 * @return
 */

    protected synchronized String
    getMessage() throws InterruptedException {
        String message = null;

        if (SocketThread_Debug) {
            System.out.println("SocketThread: getMessage.");
        }

        notifyAll();
	while (messages.size() == 0) {
            wait();
        }
        message = (String) messages.firstElement();
        messages.removeElement(message);
	return(message);
    }
 
 
/**
 * <A NAME="SD_GETSOCKETMESSAGE"></A>
 * <EM>getSocketMessage</EM>
 *
 * @return
 */
 
    String
    getSocketMessage() {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: getSocketMessage.");
        }
 
        System.out.println("SocketThread: getSocketMessage: " +
                           "Should be over-ridden by it's subclass.");
        return(null);
    }


/**
 * <A NAME="SD_GETHOST"></A>
 * <EM>getHost</EM>
 *
 * @return
 */
 
    protected String
    getHost() {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: getHost.");
        }
 
        return(host);
    }


/**
 * <A NAME="SD_GETPORT"></A>
 * <EM>getPort</EM>
 *
 * @return
 */
 
    protected int
    getPort() {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: getPort.");
        }

	return(port);
    }


/**
 * <A NAME="SD_PUTMESSAGE"></A>
 * <EM>putMessage</EM>
 *
 * @param <EM>s</EM>
 *
 * @exception InterruptedException
 */

    protected synchronized void
    putMessage(String s) throws InterruptedException {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: putMessage: " + s);
        }
 
        while (messages.size() == MAXQUEUE) {
            wait();
        }
        messages.addElement(s);
        notifyAll();
    }


/**
 * <A NAME="SD_PUTMESSAGEANDDATA"></A>
 * <EM>putMessageAndData</EM>
 *
 * @param <EM>s</EM>
 *
 * @exception InterruptedException
 */

    protected void
    putMessageAndData(String s) throws InterruptedException {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: putMessageAndData: " + s);
        }

        System.out.println("SocketThread: putMessageAndData: " +
                           "Should be over-ridden by it's subclass.");
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>message</EM>
 */

    protected void
    handleMessage(String message) {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: handleMessage:" +
                               " message: " + message);
        }

        System.out.println("SocketThread: handleMessage: " +
                           "Should be over-ridden by it's subclass.");
    }  


/**
 * <A NAME="SD_HANDLEMESSAGEANDDATA"></A>
 * <EM>handleMessageAndData</EM>
 *
 * @param <EM>message</EM>
 * @param <EM>data</EM>
 */
 
    protected void
    handleMessageAndData(String message, byte data[]) {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: handleMessageAndData:" +
                               " message: " + message +
			       " data: "    + data);
        }
 
        System.out.println("SocketThread: handleMessageAndData: " +
                           "Should be over-ridden by it's subclass.");
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM>
 */

    public void
    run() {
        if (SocketThread_Debug) {
            System.out.println("SocketThread: run.");
        }

        try {
            while (true) {
                String s = null;

		if ((s = getSocketMessage()) == null) {
		    break;
                }

                if (SocketThread_Debug) {
                    System.out.println("SocketThread: run:" +
                        " - received: " + s + " (" + s.length() + " bytes)");
                }

                putMessage(s);
                System.out.println("Inside putMessage method");


                if (isWaiting == false) {
                    while (messages.size() > 0) {
                        handleMessage(getMessage());
                        System.out.println("Handling the getMessage method");
                    }
                }
            }    
        } catch (Exception e) {
            System.out.println("SocketThread: run - exception " + e);
            if (SocketThread_Debug) {
		e.printStackTrace();
	    }
        }
    }    
}
