
/*
 * @(#)Registry.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import java.net.ServerSocket;
import java.net.Socket;
import java.util.*;

/**
 * Shared Data Registry class.
 *
 * A "registry" exists on every node that allows Shared Data connections
 * to servers on that node.  The registry on a particular node contains a
 * transient database that maps names to Shared Data objects.  When the
 * node boots, the registry database is empty.  The names stored in the
 * registry are pure and are not parsed.  A service storing itself in
 * the registry may want to prefix its name of the service by a package
 * name (although not required), to reduce name collisions in the
 * registry.
 *
 * Based on the RMI Registry class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

class
Registry extends socketSharedDataObject {

    /** Default Shared Data registry port number. */
    static final int REGISTRY_PORT = 80;
     

    /** The threads currently being run by the server. */
    private Vector connections = null;

    /** The server socket to await connections on. */
    private ServerSocket ssock;

    /** The objects currently bound by this registry. */
    private Hashtable bindings = null;


/**
 * <A NAME="SD_REGISTRY"></A>
 * <EM>Registry</EM> the constructor for the Shared Data registry.
 *
 * @param <EM>port</EM> the port number to use for messages.
 */

    public
    Registry(int port) {
        if (Registry_Debug) {
            System.out.println("Registry: constructor:" +
                               " port: " + port);
        }

        try {
            ssock       = new ServerSocket(port);
            System.out.println("ServerSocket opened at "+port);
	    bindings    = new Hashtable(101);
            connections = new Vector();
        } catch (Exception e) {
            System.out.println("Registry: constructor: exception " + e);
            if (Registry_Debug) {
		e.printStackTrace();
	    }
        }
    }


/**
 * <A NAME="SD_BIND"></A>
 * <EM>bind</EM> binds the name to the specified Shared Data object.
 *
 * @param <EM>who</EM>
 * @param <EM>name</EM> the name associated with this new object.
 */

    String
    bind(String who, String name) {
        int retval = 0;

	if (Registry_Debug) {
            System.out.println("Registry: bind:" +
                               " who: "  + who +
			       " name: " + name);
        }

	synchronized (bindings) {
	    if (bindings.get(name) != null) {
                retval = SharedData.SharedDataException.ALREADY_BOUND;
	    } else {
	        bindings.put(name, new Object());
	    }
	}
        return(T_Bind + T_Sep + who + T_Sep + retval);
    }


/**
 * <A NAME="SD_REBIND"></A>
 * <EM>rebind</EM>
 *
 * @param <EM>who</EM>
 * @param <EM>name</EM>
 *
 * @return
 */

    String
    rebind(String who, String name) {
        if (Registry_Debug) {
            System.out.println("Registry: rebind:" +
                               " who: "  + who +
                               " name: " + name);
        }

        synchronized (bindings) {
	    bindings.put(name, new Object());
	}
	return(T_Rebind + T_Sep + who + T_Sep + "0");
    }


/**
 * <A NAME="SD_UNBIND"></A>
 * <EM>unbind</EM>
 *
 * @param <EM>who</EM>
 * @param <EM>name</EM>
 *
 * @return
 */

    String
    unbind(String who, String name) {
        int retval = 0;

        if (Registry_Debug) {
            System.out.println("Registry: unbind:" +
                               " who: "  + who +
                               " name: " + name);
        }

	synchronized (bindings) {
	    if (bindings.get(name) == null) {
	        retval = SharedData.SharedDataException.NOT_BOUND;
	    } else {
	        bindings.remove(name);
	    }
	}
        return(T_Unbind + T_Sep + who + T_Sep + retval);
    }


/**
 * <A NAME="SD_LOOKUP"></A>
 * <EM>lookup</EM>
 *
 * @param <EM>who</EM>
 * @param <EM>name</EM>
 *
 * @return
 */

    String
    lookup(String who, String name) {
        int retval = 0;

        if (Registry_Debug) {
            System.out.println("Registry: lookup:" +
                               " who: "  + who +
                               " name: " + name);
        }

        synchronized (bindings) {
	    if (bindings.get(name) == null) {
	        retval = SharedData.SharedDataException.NOT_BOUND;
	    }
	}

        return(T_Lookup + T_Sep + who + T_Sep + retval);
    }


/**
 * <A NAME="SD_LIST"></A>
 * <EM>list</EM>
 *
 * @param <EM>who</EM>
 *
 * @return
 */

    String
    list(String who) {
        int         size  = 0;
        Enumeration enum  = null;
        String      reply = null;

        if (Registry_Debug) {
            System.out.println("Registry: list:" +
                               " who: " + who);
        }

	synchronized (bindings) {
	    size  = bindings.size();
            enum  = bindings.keys();
	    reply = T_List + T_Sep + who + T_Sep + "0" + T_Sep + size;
	    while ((--size) >= 0) {
	        reply += "\u0001" + (String) enum.nextElement();
	    }
	}
        return(reply);
    }


/**
 * <A NAME="SD_GETPORT"></A>
 * <EM>getPort</EM>
 *
 * @param <EM>args</EM>
 *
 * @return
 */

    private static int
    getPort(String args[]) {
        int defPort = REGISTRY_PORT;
        int length = args.length;

        if (Registry_Debug) {
            System.out.println("Registry: getPort.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-port")) {
                if (++i < length) {
                    return(Integer.parseInt(args[i]));
                }
            }    
        }
        return(defPort);
    }


/**
 * <A NAME="SD_HANDLEREQUESTS"></A>
 * <EM>handleRequests</EM>
 */

    private void
    handleRequests() {
        if (Registry_Debug) {
            System.out.println("Registry: handleRequests.");
        }

        try {
            while (true) {
                Socket sock;
		RegistryServerThread rst;
		Thread thread;

		if (Registry_Debug) {
		    System.out.println("Registry: main: Waiting...");
                }
		sock = ssock.accept();
                if (Registry_Debug) {
		    System.out.println("Registry: Got a connection.");
		}

                rst = new RegistryServerThread(sock, this);
                connections.addElement(rst);
                thread = new Thread(rst);
		thread.start();
            }
        } catch (Exception e) {
            System.out.println("Registry: handleRequests: exception " + e);
            if (Registry_Debug) {
		e.printStackTrace();
	    }
        }
    }


/**
 * <A NAME="SD_MAIN"></A>
 * <EM>main</EM> the main program to start a registry. The port number can
 * be specified on the command line.
 *
 * @param <EM>args</EM>
 */

    public static void
    main(String args[]) {  
        Registry registry = new Registry(getPort(args));

        if (Registry_Debug) {
            System.out.println("Registry: main.");
        }

	registry.handleRequests();
    }
}
