// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ActionCreateNode.java
// Classes: ActionCreateNode
// Original Author: jrobbins@ics.uci.edu
// $Id: ActionCreateNode.java,v 1.11 1997/06/10 23:42:08 jrobbins Exp $

package uci.graphedit;

import java.util.*;

/** Editor Action to create a new Perspective on a new NetNode. When
 *  this Action is executed it makes the new objects as per its
 *  arguments, and then it sets the global next mode to ModePlace so
 *  that the user can place the new node in any editor window.
 *
 * @see ModePlace
 * @see NetNode
 * @see Perspective */

public class ActionCreateNode extends Action {

  ////////////////////////////////////////////////////////////////
  // constants
  public static String DEFAULT_NODE_CLASS = "uci.graphedit.demo.SampleNode";

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** The class of the new NetNode. */
  private Class _nodeClass;

  /** The place to get default node attributes for the new NetNode. */
  private NetNode _default_node;

  ////////////////////////////////////////////////////////////////
  // constructors

  /** Construct a new Action with the given arguments for node class. */
  public ActionCreateNode(Properties args) {
    super(args);
    String className = args.getProperty("className", DEFAULT_NODE_CLASS);
    try { _nodeClass = Class.forName(className); }
    catch (java.lang.ClassNotFoundException ignore) { return; }
  }

  /** Construct a new Action with the given classes for the NetNode
   *  and its Perspective. */
  public ActionCreateNode(String nodeClassName, String modelClassName) {
    super(new Properties());
    setArg("className", nodeClassName);
    if (modelClassName != null)
      setArg("modelClassName", modelClassName);
    try { _nodeClass = Class.forName(nodeClassName); }
    catch (java.lang.ClassNotFoundException ignore) {
        System.out.println("Class " + nodeClassName + " could not be found!");
        return; }
  }

  /** Construct a new Action with the given classes for the NetNode
   *  and its Perspective, and set the global sticky mode boolean to
   *  the given value. This allows the user to place several nodes
   *  rapidly.  */
  public ActionCreateNode(String node, String model, boolean sticky) {
    this(node, model);
    setArg("shouldBeSticky", sticky ? "True" : "False");
  }

  ////////////////////////////////////////////////////////////////
  // Action API

  public String name() { return "Create Node"; }

  /** Actually instanciate the NetNode and Perspective objects and
   * set the global next mode to ModePlace */
  public void doIt(java.awt.Event e) {
    NetNode newNode;
    Object model = null;
    try { newNode = (NetNode) _nodeClass.newInstance(); }
    catch (java.lang.IllegalAccessException ignore) { return; }
    catch (java.lang.InstantiationException ignore) { return; }
    // _default_node = newNode.defaults(); /* needs-more-work */

    String modelClassName = (String)getArg("modelClassName");
    if (modelClassName != null) {
      Class modelClass;
      try { modelClass = Class.forName(modelClassName); }
      catch (java.lang.ClassNotFoundException ignore) { return; }
      try { model = modelClass.newInstance(); }
      catch (java.lang.IllegalAccessException ignore) { return; }
      catch (java.lang.InstantiationException ignore) { return; }
      Editor ed = Globals.curEditor();
      newNode.initialize(_default_node, model);
    }
    else
      newNode.initialize(_default_node, null);

    Mode placeMode = new ModePlace(newNode);

    String shouldBeSticky = (String)getArg("shouldBeSticky");
    if (shouldBeSticky == null) Globals.mode(placeMode);
    else Globals.mode(placeMode, shouldBeSticky.equals("True"));
  }

  public void undoIt() {
    System.out.println("undo is not implemented");
  }

} /* end class ActionCreateNode */
