// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: LayerComposite.java
// Classes: LayerComposite
// Original Author: jrobbins@ics.uci.edu
// $Id: LayerComposite.java,v 1.13 1997/06/10 23:42:56 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** This class implements a kind of Layer that contains other
 *  Layers. Layer's can be nested in an is-part-of tree. That tree can
 *  be walked to draw the contents of the view, find what the user
 *  clicked on, find a layer by name, save the contents to a file,
 *  etc. */

public class LayerComposite extends Layer {
  ////////////////////////////////////////////////////////////////
  // instance variables

  /** The Layer's contained within this LayerComposite. */
  protected Vector _sublayers = new Vector();

  /** In most editors one Layer is the active layer and all mouse
   *  clicks go to the contents of that layer.  For now I assume this,
   *  but I would like to avoid this assumption in the future. */
  protected Layer _activeLayer;

  ////////////////////////////////////////////////////////////////
  // constructors and related methods

  /** Construct a new LayerComposite with no sublayers. */
  public LayerComposite() { }

//   public Layer shallowCopy() {
//     LayerComposite lc = (LayerComposite) super.shallowCopy();
//     lc._sublayers = (Vector)_sublayers.clone();
//     lc._activeLayer = _activeLayer;
//     return lc;
//   }

  ////////////////////////////////////////////////////////////////
  // accessors

  /** Add a sublayer to this layer. */
  public void addSublayer(Layer lay) {
    if (findLayerNamed(lay.name()) == null) {
      _sublayers.addElement(lay);
      lay.addEditors(this.editors());
      setActive(lay);
    }
  }

  /** Remove a sublayer to this layer. */
  public void removeSublayer(Layer lay) {
    _sublayers.removeElement(lay);
    lay.removeEditors(this.editors());
    if (_activeLayer == lay) {
      if (_sublayers.size() >= 1)
	_activeLayer = (Layer) _sublayers.elementAt(0);
      else
	_activeLayer = null;
    }
  }

  /** Find a layer with the given name somewhere in the layer tree. */
  public Layer findLayerNamed(String aName) {
    Layer res = super.findLayerNamed(aName);
    if (res != null) return res;
    Enumeration layers = _sublayers.elements();
    while (layers.hasMoreElements()) {
      Layer curLayer = (Layer) layers.nextElement();
      res = curLayer.findLayerNamed(aName);
      if (res != null) return res;
    }
    return res;
  }

  /** Make one of my sublayers the active one. */
  public void setActive(Layer lay) {
    if (_sublayers.contains(lay)) _activeLayer = lay;
    else System.out.println("That layer is not one of my sublayers");
  }

  /** Reply which layer is the active one. In case LayerComposite's
   *  are nested, this works recursively. */
  public Layer activeLayer() { return _activeLayer.activeLayer(); }

  /** When an editor or some tool wants to look at all the
   *  DiagramElement's that are contained in this layer, reply the
   *  contents of my active layer. Maybe this should really reply _all_
   *  the contents of all sublayers. */
  public Vector contents() {
    if (_activeLayer == null) return null;
    return _activeLayer.contents();
    /* needs-more-work: should accumulate???? */
    /* Vector v = new Vector();
    Enumerations layers = _sublayers.elements();
    while (layers.hasMoreElements()) {
      Layer lay = (Layer) layers.nextElement();
      Vector lay_contents = lay.contents();
      if (lay_contents != null) return lay_contents;
    }  */
  }

  ////////////////////////////////////////////////////////////////
  // drawing methods

  /** Draw the contents of this LayerComposite by drawing all sublayers. */
  public void drawContents(Graphics g) {
    Enumeration layers = _sublayers.elements();
    while (layers.hasMoreElements()) {
      Layer lay = (Layer) layers.nextElement();
      lay.draw(g);
    }
  }

  ////////////////////////////////////////////////////////////////
  // Layer API

  /** When the user tries to add a new DiagramElement to a
   *  LayerComposite, pass that addition along to my active layer. */
  public void add(DiagramElement de) {
    if (_activeLayer == null) return;
    _activeLayer.add(de);
  }

  /** When the user tries to remove a new DiagramElement from a
   *  LayerComposite, pass that removal along to my active layer. */
  public void remove(DiagramElement de) {
    if (_activeLayer == null) return;
    _activeLayer.remove(de);
  }

  /** See comments above, this message is passed to my active layer. */
  public void removeAll() {
    if (_activeLayer == null) return;
    _activeLayer.removeAll();
  }

  /** See comments above, this message is passed to my active layer. */
  public Enumeration elements() {
    if (_activeLayer == null) return null;
    return _activeLayer.elements();
  }

  /** See comments above, this message is passed to my active layer. */
  public DiagramElement pick(int x, int y) {
    if (_activeLayer == null) return null;
    return _activeLayer.pick(x, y);
  }

  /** Try to find a Perspective instance that presents the given
   *  Net-level object. */
  public DiagramElement perspectiveFor(Object obj) {
    DiagramElement de = null;
    Enumeration lays = _sublayers.elements();
    while (lays.hasMoreElements()) {
        Layer sub = (Layer) lays.nextElement();
        de = sub.perspectiveFor(obj);
        if (de != null) return de;
    }
    return null;
  }

  /** See comments above, this message is passed to my active layer. */
  public void sendToBack(DiagramElement prim) {
    if (_activeLayer == null) return;
    _activeLayer.sendToBack(prim);
  }

  /** See comments above, this message is passed to my active layer. */
  public void bringForward(DiagramElement prim) {
    if (_activeLayer == null) return;
    _activeLayer.bringForward(prim);
  }

  /** See comments above, this message is passed to my active layer. */
  public void sendBackward(DiagramElement prim) {
    if (_activeLayer == null) return;
    _activeLayer.sendBackward(prim);
  }

  /** See comments above, this message is passed to my active layer. */
  public void bringToFront(DiagramElement prim) {
    if (_activeLayer == null) return;
    _activeLayer.bringToFront(prim);
  }

  /** See comments above, this message is passed to my active layer. */
  public void reorder(DiagramElement prim, int function) {
    if (_activeLayer == null) return;
    _activeLayer.reorder(prim, function);
  }

  public void addEditor(Editor ed) {
    Enumeration lays = _sublayers.elements();
    while (lays.hasMoreElements()) {
        Layer sub = (Layer) lays.nextElement();
	sub.addEditor(ed);
    }
  }

  public void removeEditor(Editor ed) {
    Enumeration lays = _sublayers.elements();
    while (lays.hasMoreElements()) {
        Layer sub = (Layer) lays.nextElement();
	sub.removeEditor(ed);
    }
  }

} /* end class LayerComposite */
