// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ModeCreate.java
// Classes: ModeCreate
// Original Author: jrobbins@ics.uci.edu
// $Id: ModeCreate.java,v 1.20 1997/06/10 23:43:03 jrobbins Exp $

package uci.graphedit;

import java.awt.*;

/** Abstract superclass for all Mode's that create new
 *  DiagramElement's. This class factors our shared code that would
 *  otherwise be duplicated in its subclasses. On a mouse down the new
 *  item is created in memory. On mouse drag the new item is resized
 *  via its createDrag() method. On mouse up the new item is
 *  officially added to the document being edited in the parent
 *  Editor, the item is selected, and the Editor is placed in the next
 *  Mode (usually ModeSelect). Subclasses override various of these
 *  event handlers to give specific behaviors, for example,
 *  ModeCreateArc handles dragging differently.
 *
 * @see ModeCreateArc
 * @see ModeCreateFigRect */

public abstract class ModeCreate extends Mode {
  ////////////////////////////////////////////////////////////////
  // instance variables

  /** Original mouse down event coordinates */
  protected int anchorX, anchorY;

  /** This holds the DiagramElement to be added to the parent Editor. */
  protected DiagramElement newItem;

  ////////////////////////////////////////////////////////////////
  // constructors

  public ModeCreate(Editor par) { super(par); }

  public ModeCreate() { super(); }

  ////////////////////////////////////////////////////////////////
  // event handlers

  private Point snapPt = new Point(0, 0);

  /** On mouse down, make a new DiagramElement in memory. */
  public boolean mouseDown(Event e, int x, int y) {
    start();
    synchronized (snapPt) {
      snapPt.move(x, y);
      parent.snap(snapPt);
      anchorX = snapPt.x;
      anchorY = snapPt.y;
    }
    newItem = createNewItem(e, anchorX, anchorY);
    return true;
  }

  /** On mouse drag, resize the new item as the user moves the
   *  mouse. Maybe the DiagramElement createDrag() method should be removed
   *  and I should call dragHandle(). That would elimiate one method
   *  from each oif several classes, but dragging during creation is
   *  not really the same thing as dragging after creation... */
  public boolean mouseDrag(Event e, int x, int y) {
    parent.damaged(newItem);
    creationDrag(x, y);
    parent.damaged(newItem);
    return true;
  }

  /** On mouse up, officially add the new item to the parent Editor
   *  and select it. Then exit this mode. */
  public boolean mouseUp(Event e, int x, int y) {
    if (newItem != null) {
      parent.damaged(newItem);
      creationDrag(x, y);
      parent.add(newItem);
      parent.selectItem(newItem);
      newItem = null;
    }
    done();
    return true;
  }

  /** The default size of a Fig if the user simply clicks instead of
   *   dragging.
   *  <A HREF="../features.html#default_initial_size">
   *  <TT>FEATURE: default_initial_size</TT></A>
   */
  protected static int _defaultWidth = 32, _defaultHeight = 32;

  /** Update the new item to reflect the new mouse position. By
   *  default let the new item set its size, subclasses may
   *  override. If the user simply clicks instead of dragging then use
   *  the default size. If the user actually drags out a Fig, then
   *  use its size as the new default size.
   *  <A HREF="../features.html#default_initial_size">
   *  <TT>FEATURE: default_initial_size</TT></A>
   *
   * @see ModeCreateFigLine#creationDrag */
  protected void creationDrag(int x, int y) {
    int snapX, snapY;
    synchronized (snapPt) {
      snapPt.move(x, y);
      parent.snap(snapPt);
      snapX = snapPt.x;
      snapY = snapPt.y;
    }
    if (anchorX == snapX && anchorY == snapY)
      ((Fig)newItem).createDrag(anchorX, anchorY,
				x + _defaultWidth,
				y + _defaultHeight,
				snapX + _defaultWidth,
				snapY + _defaultHeight);
    else {
      ((Fig)newItem).createDrag(anchorX, anchorY, x, y, snapX, snapY);
      _defaultWidth = snapX - anchorX;
      _defaultHeight = snapY - anchorY;
    }
  }

  ////////////////////////////////////////////////////////////////
  // drawing methods

  /** Draw this mode by drawing the new item. This is the only
   *  feedback that the user will get since the new item is not
   *  officially added to the Editor's document yet. */
  public void draw(Graphics g) {
    if (null != newItem) newItem.draw(g);
  }

  ////////////////////////////////////////////////////////////////
  // ModeCreate API

  /** Construct a new DiagramElement to be added to the
   *  Editor. Typically, subclasses will make a new instance of some Fig
   *  based on the given mouse down event and the state of the parent
   *  Editor (specifically, its default graphical attributes). */
  public abstract DiagramElement createNewItem(Event e, int snapX, int snapY);

} /* end class ModeCreate */

