// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ModeCreateArc.java
// Classes: ModeCreateArc
// Original Author: ics125b spring 1996
// $Id: ModeCreateArc.java,v 1.19 1997/06/10 23:43:04 jrobbins Exp $

package uci.graphedit;

import java.awt.*;

/** A Mode to interpret user input while creating an arc. Basically
 *  mouse down starts creating an arc from a source NetPort, mouse
 *  motion draws a rubberband line, mouse up finds the destination port
 *  and finishes creating the arc and makes an ArcPerspective and sends
 *  it to the back of the Layer.  */

public class ModeCreateArc extends ModeCreate {
  ////////////////////////////////////////////////////////////////
  // instance variables

  /** The NetPort where the arc is drawn from */
  private NetPort _startPort;

  /** The Fig that presents the starting NetPort */
  private Fig _startPortFig;

  /** The Perspective on the NetNode that owns the start port */
  private Perspective _sourcePerspective;

  /** The new NetArc that is being created */
  private NetArc _newArc;

  ////////////////////////////////////////////////////////////////
  // constructor

  public ModeCreateArc(Editor par) { super(par); }

  ////////////////////////////////////////////////////////////////
  // Mode API

  public String instructions() {
    return "Drag to define an arc to another port";
  }

  ////////////////////////////////////////////////////////////////
  // ModeCreate API

  /** Create the new item that will be drawn. In this case I would
   *  rather create the ArcPerspective when I am done. Here I just
   *  create a rubberband line to show during the interaction. */
  public DiagramElement createNewItem(Event e, int snapX, int snapY) {
    return new FigLine(snapX, snapY, 0, 0, Globals.prefs().rubberbandAttrs());
  }

  ////////////////////////////////////////////////////////////////
  // event handlers

  /** On mouse down determine what port the user is dragging from. The
   *  mouse down event is passed in from ModeSelect. */
  public boolean mouseDown(Event e, int x, int y) {
    DiagramElement underMouse = parent.pick(x, y);
    if (underMouse == null) { done(); return true; }
    if (!(underMouse instanceof Perspective)) { done(); return true; }
    _sourcePerspective = (Perspective) underMouse;
    _startPort = _sourcePerspective.pickPort(x, y);
    if (_startPort == null) { done(); return true; }
    _startPortFig = _sourcePerspective.getPortFig(_startPort);
    return super.mouseDown(e, x, y);
  }

  /** On mouse up, find the destination port, ask the NetArc
   *  (e.g., SampleArc) to connect the two ports. If that connection is
   *  allowed, then construct a new ArcPerspective and add it to the
   *  Layer and send it to the back. */
  public boolean mouseUp(Event e, int x, int y) {
    Class arcClass;

    DiagramElement de = parent.pick(x, y);

    if (de != null) {
      if (de instanceof Perspective) {
	Perspective destPerspective = (Perspective) de;
	/* If its a perspective, then check within the  */
	/* perspective to see if a port exists */
	NetPort foundPort = destPerspective.pickPort(x, y);

	if (foundPort != null && foundPort != _startPort) {
	  Fig destPortFig;
	  destPortFig = destPerspective.getPortFig(foundPort);
	  _newArc = _startPort.makeArcFor(foundPort);
	  if (_newArc == null) return true;

	  if (_newArc.connect(_startPort, foundPort)) {
 	    parent.net().addArc(_newArc);
	    ArcPerspective pers =
	      (ArcPerspective) parent.view().perspectiveFor(_newArc);
	    parent.add(pers); // adds it to the property sheet's universe
	    newItem.damagedIn(parent);
	    newItem = null;
	    if (pers != null) {
	      pers.reorder(ActionReorder.SEND_TO_BACK, parent.view());
	      parent.selectItem(pers);
	    }
	    done();
	    return true;
	  }
	}
      }
    }
    newItem.damagedIn(parent);
    newItem = null;
    done();
    return true;
  }

} /* end class ModeCreateArc */
