// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ModeStack.java
// Classes: ModeStack
// Original Author: jrobbins@ics.uci.edu
// $Id: ModeStack.java,v 1.7 1997/06/10 23:43:14 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.util.*;
import uci.util.*;

/** ModeStack is a composite Mode that is made up of several other
 *  Modes.  Events are passed to the submodes for handling.  The
 *  submodes are prioritized according to their order on the stack,
 *  i.e., the last mode added gets the first chance to handle an
 *  Event.  */

public class ModeStack extends Mode {

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** The stack of Mode's that are all active simultainously, the
   *  order of Mode's on the stack is their priority, i.e., the
   *  topmost Mode gets the first chance to handle an incoming
   *  Event. Needs-More-Work: this is a time critical part of the
   *  system and should be faster, use an array instead of a Vector.*/
  private Vector _subModes = new Vector();

  ////////////////////////////////////////////////////////////////
  // constructors

  /**  Construct an empty ModeStack. */
  public ModeStack() {  }

  /**  Construct a ModeStack with the given mode(s). */
  public ModeStack(Mode m1) { push(m1); }

  /**  Construct a ModeStack with the given mode(s). */
  public ModeStack(Mode m1, Mode m2) { push(m1); push(m2); }

  ////////////////////////////////////////////////////////////////
  //  accessors

  /** Reply the stack of Mode's. */
  public Vector modes() { return _subModes; }

  /** Set the entire stack of Mode's. */
  public void modes(Vector newModes) { _subModes = newModes; }

  /** Reply the top (first) Mode. */
  public Mode top() {
    if (_subModes.isEmpty()) return null;
    else return (Mode)_subModes.lastElement();
  }

  /** Add the given mode to the composite iff another instance
   * of the same class is not already on the stack. */
  public void push(Mode newMode) {
    Class newModeClass = newMode.getClass();
    Enumeration subs = _subModes.elements();
    while (subs.hasMoreElements()) {
      Mode m = (Mode) subs.nextElement();
      if (m.getClass() == newModeClass) return;
    }
    _subModes.addElement(newMode);
  }

  /** Remove the topmost Mode iff it can exit. */
  public Mode pop() {
    if (_subModes.isEmpty()) return null;
    Mode res = top();
    if (res.canExit()) _subModes.removeElement(res);
    return res;
  }

  /** Remove all Modes that can exit. */
  public void popAll() {
    while (!_subModes.isEmpty() && top().canExit())
      _subModes.removeElement(top());
  }

  ////////////////////////////////////////////////////////////////
  // event handlers

  /** Pass the event to each mode in the stack until one handles it. */
  public boolean handleEvent(Event e) {
    boolean res = super.handleEvent(e);
    for (int i = _subModes.size() - 1; i >= 0 && !res; --i) {
      Mode m = ((Mode)_subModes.elementAt(i));
      //DebugLog.log("DebugDispatch2", "passing event to mode #" + i);
      try { res = m.handleEvent(e); }
      catch (java.lang.Throwable ex) {
	System.out.println("While passing event " + e.toString() +
			   " to mode " + m.toString() +
			   " the following error occured:");
	ex.printStackTrace();
      }
    }
    return res;
  }

  /** Give each Mode a chance to prehandle the event. */
  public void preHandleEvent(Event e) {
    checkModeTransitions(e);
    for (int i = 1; i < _subModes.size(); ++i) {
      //DebugLog.log("DebugDispatch2", "prehandeling event with mode #" + i);
      ((Mode)_subModes.elementAt(i)).preHandleEvent(e);
    }
    super.preHandleEvent(e);
  }

  /** Give each Mode a chance to posthandle the event. */
  public void postHandleEvent(Event e) {
    super.postHandleEvent(e);
    for (int i = _subModes.size() - 1; i >= 0; --i) {
      //DebugLog.log("DebugDispatch2", "posthandeling event with mode #" + i);
      ((Mode)_subModes.elementAt(i)).postHandleEvent(e);
    }
  }

  ////////////////////////////////////////////////////////////////
  // mode transitions

  /** Check for events that should cause transitions from one Mode to
   *  another or otherwise change the ModeStack. Really this should be
   *  specified in a subclass of ModeStack, because Modestack should
   *  not make assumptions about the look-and-feel of all future
   *  applications.  Needs-More-Work: I would like to put the
   *  transition from ModeSelect to ModeModify here, but there are too
   *  many interactions, so that code is still in ModeSelect. */
  public void checkModeTransitions(Event e) {
    if (top() instanceof ModeSelect && e.id == Event.MOUSE_DOWN) {
      DiagramElement underMouse = parent.pick(e.x, e.y);
      if (underMouse instanceof Perspective) {
	NetPort startPort = ((Perspective) underMouse).pickPort(e.x, e.y);
	if (startPort != null) {
	  parent.mode(new ModeCreateArc(parent));
	  return;
	}
      }
    }
  }

  ////////////////////////////////////////////////////////////////
  // drawing methods

  /** Draw each mode in the stack: bottom to top. */
  public void draw(Graphics g) {
    Enumeration modes = _subModes.elements();
    while (modes.hasMoreElements()) {
      Mode m = (Mode) modes.nextElement();
      m.draw(g);
    }
  }

} /* end class ModeStack */
