// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: NetPort.java
// Classes: NetPort
// Original Author: ics125b spring 1996
// $Id: NetPort.java,v 1.9 1997/06/10 23:43:17 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.util.*;
import java.io.*;

/** This class models a port in our underlying connected graph model.
 *  <A HREF="../features.html#graph_representation_ports">
 *  <TT>FEATURE: graph_representation_ports</TT></A>
 */

public class NetPort extends NetPrimitive {

  ////////////////////////////////////////////////////////////////
  // constants
  public static String DEFAULT_ARC_CLASS = "uci.graphedit.demo.SampleArc";

  /** Constants useful for determining what side (north, south, east,
   * or west) a port is located on. Maybe this should really be in
   * Perspective. */
  public static final double ang45 = Math.PI / 4;
  public static final double ang135 = 3*Math.PI / 4;
  public static final double ang225 = 5*Math.PI / 4;
  public static final double ang315 = 7*Math.PI / 4;

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** The NetArc's that are connected to this port. */
  protected Vector arcList;

  /** The NetNode that this port is a part of. */
  protected NetNode parentNode;

  ////////////////////////////////////////////////////////////////
  // constructors

  /** Construct a new NetPort with the given parent node and no arcs. */
  public NetPort(NetNode parent) {
    parentNode = parent;
    arcList = new Vector();
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  /** Reply the NetNode that owns this port. */
  public NetNode parentNode() { return parentNode; }

  /** Reply a vector of NetArcs that are connected here. */
  public Vector arcs() { return arcList; }

  ////////////////////////////////////////////////////////////////
  // net-level operations

  /** Add an arc to the list of arcs connected to this port. Called
   *  when the user defines a new arc. Normally, you would not call
   *  this directly, you would call NetArc#connect(). */
  public void addArc(NetArc arc) { arcList.addElement(arc); }

  /** Remove an arc from the list of arcs connected to this
   *  port. Called when the user deletes an arc. Normally, you would
   *  not call this directly, you would call NetArc#dispose().*/
  public void removeArc(NetArc arc) { arcList.removeElement(arc); }

  ////////////////////////////////////////////////////////////////
  // Editor API

  /** Remove this port from the underlying connected graph model and
   *  dispose all arcs connected to it. */
  public void dispose() {
    Enumeration arcs = arcList.elements();
    while (arcs.hasMoreElements()) {
      NetArc a = (NetArc) arcs.nextElement();
      a.dispose();
    }
  }

  ////////////////////////////////////////////////////////////////
  // net-level hooks

  /** Application specific hook that is called after a successful
   *  connection. */
  public void postConnect(NetPort anotherPort) {
    parentNode.postConnect(anotherPort.parentNode(), this, anotherPort);
  }

  /** Application specific hook that is called after a
   *  disconnection. (for now, all disconnections are assumed
   *  legal). */
  public void postDisconnect(NetPort anotherPort) {
    parentNode.postDisconnect(anotherPort.parentNode(), this, anotherPort);
  }

  /** reply the java Class to be used to make new arcs. This is a
   *  utility function called from NetPort#makeArcFor */
  protected Class defaultArcClass(NetPort otherPort) {
    try { return Class.forName(DEFAULT_ARC_CLASS); }
    catch (java.lang.ClassNotFoundException ignore) { return null; }
  }

  /** reply a new NetArc from this port to the given NetPort. */
  public NetArc makeArcFor(NetPort otherPort) {
    Class arcClass;
    NetArc arc;
    arcClass = defaultArcClass(otherPort);
    if (arcClass == null) return null;
    try { arc = (NetArc) arcClass.newInstance(); }
    catch (java.lang.IllegalAccessException ignore) { return null; }
    catch (java.lang.InstantiationException ignore) { return null; }
    return arc;
  }

  ////////////////////////////////////////////////////////////////
  // net-level constraints

  /** Reply true if this port can legally be connected to the given
   *  port. Subclasses may implement this to reflect application
   *  specific connection constraints. By default, each port just
   *  defers that decision to its parent NetNode. By convention, your
   *  implementation should return false if super.canConnectTo() would
   *  return false (i.e., deeper subclasses get more constrained). I
   *  don't know if that is a good convention. */
  public boolean canConnectTo(NetPort anotherPort) {
    return parentNode.canConnectTo(anotherPort.parentNode(),
				   anotherPort,
				   this);
  }

  ////////////////////////////////////////////////////////////////
  // diagram-level operations

  /** Reply the port's sector within the current view.  This version
   *  works precisely with square perspectives the angxx constants
   *  should be removed and calculated by the port if non-square
   *  perspectives will be used.
   *
   *  <pre>Sectors
   *		      \  1   /
   *		       \    /
   *		        \  /
   *		     2   \/   -2
   *			 /\
   *		        /  \
   *		       /    \
   *		      /  -1  \ </pre>
   **/

  public int getPortSector(Perspective nodePers) {
    Rectangle nodeBBox = nodePers.getBoundingBox();
    Fig portPaint = nodePers.getPortFig(this);
    Rectangle portBBox = portPaint.getBoundingBox();
    int nbbCenterX = nodeBBox.x + nodeBBox.width / 2;
    int nbbCenterY = nodeBBox.y + nodeBBox.height / 2;
    int pbbCenterX = portBBox.x + portBBox.width / 2;
    int pbbCenterY = portBBox.y + portBBox.height / 2;

    if (portPaint != null) {
      int dx = (pbbCenterX - nbbCenterX) * nodeBBox.height;
      int dy = (pbbCenterY - nbbCenterY) * nodeBBox.width;
      double dist = Math.sqrt(dx * dx + dy * dy);
      double ang;
      if (dy > 0) ang = Math.acos(dx / dist);
      else ang = Math.acos(dx / dist) + Math.PI;

      if (ang < ang45) return 2;
      else if (ang < ang135) return 1;
      else if (ang < ang225) return -2;
      else if (ang < ang315) return -1;
      else return 2;
    }
    return -1;
  }

} /* end class NetPort */

