// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: Selection.java
// Classes: Selection
// Original Author: jrobbins@ics.uci.edu
// $Id: Selection.java,v 1.17 1997/06/10 23:43:29 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;
import uci.util.*;

/** This class represents the "selection" object that is used when you
 *  select one or more Figures in the drawing window. Selection's
 *  (should) handle the display of handles or whatever graphics
 *  indicate that something is selected, and they process events to
 *  manipulate the selected DiagramElement. Needs-More-Work: in a
 *  earlier version of this design DiagramElement's did all of that
 *  work themselves which led to much code duplication. Unfortuanately
 *  some of that code is still left over. Specificially,
 *  DiagramElement's still have methods to draw their own handles.<p>
 *  <A HREF="../features.html#selections">
 *  <TT>FEATURE: selections</TT></A>
 *
 * @see DiagramElement
 * @see DiagramElement#drawSelected */

public abstract class Selection extends EventHandler implements GEF {

  /** Construct a new selection. Empty, subclases can override */
  public Selection() { }

  /** Reply true if this selection contains the given DiagramElement */
  public abstract boolean contains(DiagramElement de);

  /** Reply true if nothing is selected. */
  public boolean isEmpty() { return size() == 0; }

  /** Reply the number of DiagramElement's selected */
  public abstract int size();

  /** Draw something that indicates a selection to the user */
  public abstract void draw(Graphics g);

  /** Some action is about to change the selected item, mark its
   *  region as damaged .*/
  public abstract void startTrans();

  /** Some action has just changes the selected item, mark its region
   *  as damaged .*/
  public abstract void endTrans();

  /** Get and set the graphical attributes of the selected
   * DiagramElement(s).
   *
   * @see DiagramElement#get
   * @see DiagramElement#put */
  public abstract Object get(String key);
  public abstract boolean put(String k, Object v);
  public boolean put(Hashtable newAttrs) {
    Enumeration cur = newAttrs.keys();
    while (cur.hasMoreElements()) {
      String key = (String) cur.nextElement();
      Object val = newAttrs.get(key);
      put(key, val);
    }
    return true;
  }

  /** Reply the position of the Selection. That is defined to be the
   *  upper left corner of my bounding box. */
  public Point position() {
    Rectangle bbox = getBoundingBox();
    return new Point(bbox.x, bbox.y);
  }

  public abstract boolean locked();

  /** This Selection needs to be redrawn. */
  public void damagedIn(Editor ed) { ed.damaged(this); }

  /** Reply true if the given mouse coordinates are inside or near
   *  enough to this selection that we consider the user to have
   *  clicked on it. */
  public abstract boolean inside(int x, int y);
  public final boolean inside(Point pnt) { return inside(pnt.x, pnt.y); }

  /** Find which handle the user clicked on, or return -1 if none. */
  public final int pickHandle(Point p) { return pickHandle(p.x, p.y); }
  public abstract int pickHandle(int x, int y);

  /** Reorder the selected DiagramElement(s). */
  public abstract void reorder(int function, Layer view);

  /** Align the selected DiagramElement(s), uses align(Rectangle, int,
   *  Editor). */
  public abstract void align(int direction);

  /** Align the selected DiagramElement(s). */
  public abstract void align(Rectangle r, int direction, Editor ed);

  /** Offset the selected DiagramElement(s). */
  public abstract void translate(int dx,int dy);

  /** Remove the selected DiagramElement(s) from the given Editor. */
  public abstract void removeFrom(Editor ed);

  /** Dispose of the selected DiagramElement(s). */
  public abstract void dispose(Editor ed);

  /** Move one of the handles of a selected DiagramElement. */
  public abstract void dragHandle(int mx, int my, int an_x,int an_y, Handle h);

  /** Reply the bounding box of the selection, including any handles. */
  public abstract Rectangle getBoundingBox();

  /** Reply the bounding box of the selected DiagramElements, does not
   *  include space used by handles. */
  public abstract Rectangle getContentBoundingBox();

  public abstract void changedProp(String key);

} /* end class Selection */

