// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: SelectionMultiple.java
// Classes: SelectionMultiple
// Original Author: jrobbins@ics.uci.edu
// $Id: SelectionMultiple.java,v 1.17 1997/06/10 23:43:31 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** This class handles multiple selections. It is basically a
 *  collection of SelectionSingle instances. Most of its operations
 *  just dispatch the same operation to each of the SelectionSingle
 *  instances in turn.
 *
 * @see SelectionSingle
 */

public class SelectionMultiple extends Selection {

  /** The collection of SelectionSingle instances */
  protected Vector _contents;

  public SelectionMultiple() {
    _contents = new Vector();
  }

  /** Add a new selection to the collection of selections */
  public void addElement(Selection s) { _contents.addElement(s); }
  public void addElement(DiagramElement de) {
    _contents.addElement(de.selectionObject());
  }
  public void addAllElements(Vector v) {
    Enumeration eles = v.elements();
    while(eles.hasMoreElements()) {
      DiagramElement de = (DiagramElement) eles.nextElement();
      addElement(de);
    }
  }

  /** Select some objects if they were not selected already, and
   * deslect others that were already selected. This is normally
   * called from ModeSelect when the shift key is held down.
   * @see ModeSelect
   * @see Editor
   */
  public void toggleAllElements(Vector v) {
    Enumeration eles = ((Vector)v.clone()).elements();
    while(eles.hasMoreElements()) {
      DiagramElement de = (DiagramElement) eles.nextElement();
      if (contains(de)) removeElement(de);
      else addElement(de);
    }
  }

  /** Deselect the given DiagramElement */
  public void removeElement(Selection s) { _contents.removeElement(s); }
  public void removeElement(DiagramElement de) {
    Enumeration eles = ((Vector)_contents.clone()).elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (sel.contains(de)) removeElement(sel);
    }
  }

  /** Deselect everything */
  public void removeAllElements() { _contents.removeAllElements(); }

  /** Reply true if the given selection instance is part of my
   * collection */
  public boolean contains(Selection s) { return _contents.contains(s); }

  /** Reply true if the given DiagramElement is selected by any of my
   * selection objects */
  public boolean contains(DiagramElement de) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (sel.contains(de)) return true;
    }
    return false;
  }

  public boolean locked() {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (sel.locked()) return true;
    }
    return false;
  }

  /** Reply the number of selected DiagramElement's. This assumes that
   * this collection holds only SelectionSingle instances and each of
   * those holds one DiagramElement */
  public int size() { return _contents.size(); }

  public Vector contents() { return _contents; }

  /** Reply the collection of all selected DiagramElement's */
  public Vector deContents() {
    Vector des = new Vector(_contents.size());
    Enumeration eachSel = _contents.elements();
    while (eachSel.hasMoreElements()) {
      SelectionSingle sel = (SelectionSingle) eachSel.nextElement();
      des.addElement(sel.content());
    }
    return des;
  }

  /** Start a transaction that damages all selected DiagramElement's */
  public void startTrans() {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.startTrans();
    }
  }

  /** End a transaction that damages all selected DiagramElement's */
  public void endTrans() {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.endTrans();
    }
  }

  /** Draw all selection objects */
  public void draw(Graphics g) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.draw(g);
    }
  }

  /** Reply the first non-null value of the given graphical attribute */
  public Object get(String key) {
    Object obj;
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (null != (obj = sel.get(key)))
	return obj;
    }
    return null;
  }

  /** Set the given graphical attribute for all selected objects */
  public boolean put(String k, Object v) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.put(k, v);
    }
    return true; //needs-more-work
  }

  /** When the SelectionMultiple is damaged, that implies that each
  * SelectionSingle should be damaged. */
  public void damagedIn(Editor ed) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.damagedIn(ed);
    }
  }

  /** Reply true iff the given point is inside one of the selected
   * DiagramElement's */
  public boolean inside(int x, int y) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (sel.inside(x, y)) return true;
    }
    return false;
  }

  public Rectangle getBoundingBox() {
    Rectangle r = null;
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (null == r) r = sel.getBoundingBox();
      else r.add(sel.getBoundingBox());
    }
    return r;
  }

  public Rectangle getContentBoundingBox() {
    Rectangle r = null;
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (null == r) r = sel.getContentBoundingBox();
      else r.add(sel.getContentBoundingBox());
    }
    return r;
  }

  /** Align the selected DiagramElement's relative to each other */
  /* needs-more-work: more of this logic should be in ActionAlign */
  public void align(int direction) {
    Selection sel;
    int size = _contents.size();
    Editor ed = Globals.curEditor();
    Rectangle bbox = getContentBoundingBox();

    for (int i=0; i<size; i++) {
      sel = (Selection) _contents.elementAt(i);
      sel.align(bbox, direction, ed);
    }
  }

  public void align(Rectangle r, int direction, Editor ed) {
    int size = _contents.size();
    Selection sel;
    for (int i=0; i<size; i++) {
      sel = (Selection) _contents.elementAt(i);
      sel.align(r, direction, ed);
    }
  }

  /** When multiple selections are sent to back, each of them is sent
   * to back. */
  public void reorder(int function, Layer view) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.reorder(function, view);
    }
  }

  /** When multiple selections are moved, each of them is moved */
  public void translate(int dx,int dy) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.translate(dx, dy);
      sel.changedProp(pBBOX);
    }
  }

  /** If only one thing is selected, then it is possible to mouse on
   * one of its handles, but if multiple things are selected, users
   * can only drag the objects around */
  /* needs-more-work: should take on more of this responsibility */
  public int pickHandle(int x, int y) {
    if (size() == 1) {
      int hp = ((Selection) _contents.firstElement()).pickHandle(x, y);
      return hp;
    }
    else {
      return -1;
    }
  }

  /** If only one thing is selected, then it is possible to mouse on
   * one of its handles, but if multiple things are selected, users
   * can only drag the objects around */
  public void dragHandle(int mx, int my, int an_x,int an_y, Handle h) {
    if (size() == 1) {
      Selection sel =  ((Selection) _contents.firstElement());
      sel.dragHandle(mx, my, an_x, an_y, h);
      sel.changedProp(pBBOX);
    }
    else {
      /* do nothing */
    }
  }

  /** When a multiple selection is removed, each selection is removed */
  public void removeFrom(Editor ed) {
    Enumeration eles = ((Vector)_contents.clone()).elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.removeFrom(ed);
    }
  }

  /** When a multiple selection is disposed, each selection is disposed */
  public void dispose(Editor ed) {
    Enumeration eles = ((Vector)_contents.clone()).elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.dispose(ed);
    }
  }

  /** When an event is passed to a multiple selection, will only
   *  handle key events if there is a single selected item. */
  public boolean keyDown(Event e,int key) {
    //NRS: will only handle key events if there is a single selected item.
    if(_contents.size() == 1)
      return ((Selection)_contents.elementAt(0)).keyDown(e, key);
    else
      return false;
    //NRS: the rest of these methods should be fixed, but I think
    //that it is fine for now because there are no other DiagramElements
    //that consume anything other than key events.

    //  Enumeration eles = ((Vector)_contents.clone()).elements();
    //  while(eles.hasMoreElements()) {
    //    Selection sel = (Selection) eles.nextElement();
    //    if (sel.keyDown(e, key)) return true;
    //  }
    //  return false;
  }

  public boolean mouseMove(Event e,int x,int y) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (sel.mouseMove(e, x, y)) return true;
    }
    return false;
  }

  public boolean mouseDrag(Event e,int x,int y) {
    Enumeration eles = _contents.elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (sel.mouseDrag(e, x, y)) return true;
    }
    return false;
  }

  public boolean mouseDown(Event e,int x,int y) {
    Enumeration eles = ((Vector)_contents.clone()).elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (sel.mouseDown(e, x, y)) return true;
    }
    return false;
  }

  public boolean mouseUp(Event e,int x,int y) {
    Enumeration eles = ((Vector)_contents.clone()).elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      if (sel.mouseUp(e, x, y)) return true;
    }
    return false;
  }

  ////////////////////////////////////////////////////////////////
  // notifications and updates

  public void changedProp(String key) {
    Enumeration eles = ((Vector)_contents.clone()).elements();
    while(eles.hasMoreElements()) {
      Selection sel = (Selection) eles.nextElement();
      sel.changedProp(key);
    }
  }

} /* end class SelectionMultiple */
