// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: SelectionSingle.java
// Classes: SelectionSingle
// Original Author: jrobbins@ics.uci.edu
// $Id: SelectionSingle.java,v 1.16 1997/06/10 23:43:32 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** Shared implementations of methods for all kinds of Selections that
 *  just select one thing.
 *  This is an abstract class, subclasses define the look and behavior
 *  of actual selections.
 *
 * @see SelectionMultiple
 * @see SelectionHandle
 * @see SelectionBox
 */

public abstract class SelectionSingle extends Selection {

  /** _content refers to the DiagramElement that is selected */
  protected DiagramElement _content;

  /** Construct a new selection object that refers to the given
   * DiagramElement
   */
  public SelectionSingle(DiagramElement de) {
    if (null == de) throw new java.lang.NullPointerException();
    _content = de;
  }

  /** Reply true iff this selection contains the given DiagramElement */
  public boolean contains(DiagramElement de) { return de == _content; }

  /** Reply the DiagramElement that was selected */
  public DiagramElement content() { return _content; }

  public boolean locked() { return content().locked(); }

  /** Tell the content to start a transaction that causes damage */
  public void startTrans() { content().startTrans(); }

  /** Tell the content to end a transaction that causes damage */
  public void endTrans() { content().endTrans(); }

  /** This type of selection always refers to 1 selected DiagramElement */
  public int size() { return 1; }

  /** Draw the handles or selection box or whatever is appropriate */
  public void draw(Graphics g) {
    // needs-more-work: should manage Handle objects here
    _content.drawSelected(g);
  }

  /** Reply one of the graphical attributes of the selected DiagramElement */
  public Object get(String key) {
    return _content.get(key);
  }

  /** Set one of the graphical attributes of the selected DiagramElement */
  public boolean put(String key, Object value) {
    return _content.put(key, value);
  }

  /** reply the position of the selected DiagramElement */
  public Point position() { return _content.position(); }

  /** This selection object needs to be redrawn, register its damaged
   * area within the given Editor */
  public void damagedIn(Editor ed) { _content.damagedIn(ed); }

  /** reply true if the given point is inside this selection */
  public boolean inside(int x, int y) { return _content.inside(x, y); }

  /** Tell the selected DiagramElement to move to front or back, etc. */
  public void reorder(int function, Layer view) {
    view.reorder(_content, function);
  }

  /** Do nothing because alignment only makes sense for multiple
   * selections */
  public void align(int direction) { /* do nothing */ }

  public void align(Rectangle r, int direction, Editor ed) {
    _content.align(r, direction, ed);
    //changedProp(pBBOX);
  }

  /** When the selection is told to move, move the selected
   * DiagramElement */
  public void translate(int dx,int dy) {
    _content.translate(dx, dy);
    //changedProp(pBBOX);
  }

  /** The bounding box of the selection is the bbox of the contained
   * DiagramElement. */
  public Rectangle getContentBoundingBox() {
    return _content.getBoundingBox();
  }

  /** Reply an integer handle number for the handle under the given
   * mouse point. Normally called from ModeModify.
   * @see ModeModify
   * needs-more-work: should take on more of this responsibility */
  public int pickHandle(int x, int y) { return _content.pickHandle(x, y); }

  /** Drag a handle on the selected object.  Normally called from ModeModify.
   * @see ModeModify
   */
  public void dragHandle(int mx, int my, int an_x,int an_y, Handle h) {
    _content.dragHandle(mx, my, an_x, an_y, h);
    //_content.changedProp(pBBOX);
  }

  /** If the selection is being deleted, the selected object is
   * deleted also. This is different from just deselecting the
   * selected DiagramElement, to do that use one of the deselect
   * operations in Editor
   * @see Editor#deselectItem
   */
  public void removeFrom(Editor ed) {
    _content.removeFrom(ed);
  }

  /** If the selection is being disposed, the selected object is
   * disposed also. This is different from just deselecting the
   * selected DiagramElement, to do that use one of the deselect
   * operations in Editor
   * @see Editor#deselectItem
   */
  public void dispose(Editor ed) {
    _content.dispose(ed);
  }

  /** Pass any events along to the selected DiagramElement.
   * Subclasses of SelectionSingle may reimplement this to add
   * functionality.
   */
  public boolean keyDown(Event e,int key) {
    return _content.keyDown(e, key);
  }

  public boolean mouseMove(Event e,int x,int y) {
    return _content.mouseMove(e, x, y);
  }

  public boolean mouseDrag(Event e,int x,int y) {
    return _content.mouseDrag(e, x, y);
  }

  public boolean mouseDown(Event e,int x,int y) {
    return _content.mouseDown(e, x, y);
  }

  public boolean mouseUp(Event e,int x,int y) {
    return _content.mouseUp(e, x, y);
  }

  ////////////////////////////////////////////////////////////////
  // notifications and updates
  public void changedProp(String key) {
    _content.changedProp(key);
  }

} /* end class SelectionSingle */
