package webflow.backend;

import java.io.*;
import java.util.*;

/**
* the CMreply class is used when sending requests to the ConnectionManager 
* class. It extends the Message class. It stores a fewitems in it: 
* - the status (one of an enumeration: OK or ERROR)
* - a description (if an error occured)
* For the description of the communication protocol which CMreply
* implements look under communication protocols. 
*/
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class CMreply extends Message {
  /** 
  * an enumeration of available statuses that the ConnectionManager might 
  * return 
  */
  public static final int ERROR = 0, OK = 1;
  int stat;
  String des;

  /** 
   * void constructor used by the receiving side to create an empty 
   * CMreply object to get the incomming message
   */
  public 
  CMreply() {
    stat=0;
    des= null;      
  }
 
  /** 
   * constructor used to implement replies to ESTABLISH and RECEIVE
   * requests (that need only a status)
   * @param status - int: return status
   */
  CMreply(int status) {
    stat=status;
    des = null;
  }

  /**
   * constructor used to reply with error. 
   * @param status - int: return status(ERROR)
   * @param description - String: error description
   */
  CMreply(int status, String description) {
    stat = status;
    des = description;
  }

  /** 
   * return the status stored in the object
   * @return int: status stored in the reply object
   */
  public int getStatus() {
    return stat;
  }
  
  /**
   * return the error description stored in the object
   * @return String: description of the error stored in the object
   *                 null if no error
   */
  public String getDescription() {
    return des;
  }
  
  /**
   * read the CMreply from the input stream 'in'. 
   * @param in: an InputStream from which the representation for the 
   * 	       CMreply will be read. 
   * @return true if the CMreply was read, false if some error occured. 
   */
  public boolean fromStream(InputStream in) 
  {
    boolean done = true;
    int line ;
    try
      {
	DataInputStream din = new DataInputStream(in);
	stat = din.readInt();
	
	if(din.readBoolean()) {
	  int bufsize = din.readInt();
	  byte[] b = new byte[bufsize];
	  din.readFully(b,0,bufsize);
	  des = new String(b,0,0,bufsize); 
       	}
	else
	  des = null;
	
	done=true;
      }
    catch (IOException e)
      {
	stat =0;
	done =false;
      }
    return done; 
  }
  
  /**
   * write the CMreply to the output stream 'out'. 
   * @param out: an OutputStream to which the representation for the CMreply 
   * 		will be written. 
   * @return true if the CMreply was written, false if some error occured. 
   */
  public boolean toStream(OutputStream out) {
    boolean done;
    try {
      DataOutputStream dout = new DataOutputStream(out);
      dout.writeInt(stat);

      if(des == null)
	dout.writeBoolean(false);
      else {
	dout.writeBoolean(true);
	dout.writeInt(des.length());
	dout.writeBytes(des);      
      }

      dout.flush();
      done = true;
    }
    catch(IOException e)
      { done = false; }
    return done;
  }
  
  /* test stub */
  public static void main(String[] argv)
  {
    CMreply reply = new CMreply(CMreply.ERROR, "blah");
    if (argv[0].equals("give"))
      {
	//System.out.println("stat:" + reply.getStatus());
      }
    else if(argv[0].equals("from"))
      {	
	CMreply reply1 = new CMreply(CMreply.OK);
	//System.out.println("Stat: " + reply1.getStatus());
//	System.out.println("Desc: " + reply1.getDescription());
	if(!reply1.fromStream(System.in))
          System.out.println("error  occured");
//	System.out.println("Stat: " + reply1.getStatus());
//	System.out.println("Desc: " + reply1.getDescription());
      }
    else if(argv[0].equals("to"))
      reply.toStream(System.out);
  }   
}
