package webflow.backend;

import java.io.*;

/**
* the CMrequest class is used when sending requests to the ConnectionManager 
* class. It extends the Message class. It stores a few items in it: 
* - the command (one of an enumeration: ESTABLISH, or RECEIVE)
* - two IDs refering to ports that will  be connected
* - the hostname that will receive the connection
* - a host and port address of the caller, identifying a socket at which 
*   the caller will be listening
* For the description of the communication protocol which CMrequest 
* implements look under communication protocols.  
*/ 
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class CMrequest extends Message {
  /** 
  * an enumeration of available commands to be sent through a CMrequest
  */
  public static final int ESTABLISH = 0, RECEIVE = 1;

  int cmd; 
  ID from, to;
  String dest, caller;
  int portDest, portCaller;

  /** 
  * void constructor used by the receiving side to create an empty 
  * CMrequest object to get the incomming message
  */
  CMrequest() {
    cmd = portDest = portCaller = 0;
    from = to = null;
    dest = caller = null;
  }

  /**
  * constructor used to implement the ESTABLISH command
  * @param command - command requested
  * @param fromPort - ID: from which port to establish the connection
  * @param toPort - ID: to which port to establish the connection
  * @param toHost - String: hostname of the host at which the toPort is
  *			    located
  * @param callerHost - String: hostname of the calling host
  * @param destPort - Int: port number on which the host (at which the toPort
  *                         is located) is listening
  * @param callerPort - Int: port number on which the calling host is 
  *			    listening
  */
  public
  CMrequest(int command, ID fromPort, ID toPort, String toHost, 
	    String callerHost, int destPort, int callerPort) {
    cmd = command;
    from = fromPort;
    to = toPort;
    dest = toHost;
    portDest = destPort;
    caller = callerHost;
    portCaller = callerPort;
  }

  /**
  * constructor used to implement the RECEIVE command
  * @param command - command requested
  * @param fromPort - ID: from which port to establish the connection
  * @param toPort - ID: to which port to establish the connection
  * @param callerHost - String: hostname of the calling host
  * @param callerPort - Int: port number on which the calling host is 
  *			     listening
  */
  CMrequest(int command, ID fromPort, ID toPort, String callerHost, 
	    int callerPort) {
    cmd = command;
    from = fromPort;
    to = toPort;
    dest = null;
    portDest = 0;
    caller = callerHost;
    portCaller = callerPort;
  }

  /**
  * return the command stored in CMrequest
  * @return int: value of the command stored in CMrequest
  */
  public int getCommand() {
    return cmd;
  }

  /** 
  * return the fromPort parameter stored in CMrequest
  * @return ID stored in the 'from' parameter in CMrequest
  */
  public ID getFromPort() {
    return from;
  }

  /**
  * return the toPort parameter stored in CMrequest
  * @return ID stored in the 'to' parameter in CMrequest
  */
  public ID getToPort() {
    return to;
  }

  /**
  * return the 'toHost' parameter stored in CMrequest
  * @return String: the toHost stored in the object
  */
  public String getToHost() {
    return dest;
  }

  /** 
  * return the 'callerHost' parameter stored in the object
  * @return String: the callerHost parameter
  */
  public String getCallerHost() {
    return caller;
  }

  /** 
  * return the 'callerPort' parameter stored in the object
  * @return int: the callerPort parameter
  */
  public int getCallerPort() {
    return portCaller;
  }

  /** 
  * return the 'destPort' parameter stored in the object
  * @return int: the destPort parameter
  */
  public int getDestinationPort() {
    return portDest;
  }

  /**
  * read the CMrequest from the input stream 'in'. 
  * @param in: an InputStream from which the representation for the 
  * 	       CMrequest will be read. 
  * @return true if the CMrequest was read, false if some error occured. 
  */
  public boolean fromStream(InputStream in) {
    boolean rett;

    try
      {
	DataInputStream din = new DataInputStream(in);

	// 1: command, an integer
	cmd = din.readInt();

	// 2: possible ID of from port
	if(din.readBoolean()) {
	  from = new ID();
	  from.fromStream(din);
	}
	else
	  from = null;

	// 3: possible ID of to port
	if(din.readBoolean()) {
	  to = new ID();
	  to.fromStream(din);
	}
	else 
	  to = null;

	// 4: posible String destination
	if(din.readBoolean()) {
	  int bufsize = din.readInt();
	  byte[] b = new byte[bufsize];
	  din.readFully(b,0,bufsize);
	  dest = new String(b,0,0,bufsize); 	  
	}
	else
	  dest = null;

	// 5: possible String caller address
	if(din.readBoolean()) {
	  int bufsize = din.readInt();
	  byte[] b = new byte[bufsize];
	  din.readFully(b,0,bufsize);
	  caller = new String(b,0,0,bufsize); 	  
	}
	else
	  caller = null;

	// 6: destination port, integer
	// read the two ports: dest and caller
	portDest = din.readInt();
	// 7: caller port, integer
	portCaller = din.readInt();

	rett = true;
      }
    catch(IOException e) 
      {
	from = to = null;
	caller = dest = null;
	rett = false; 
      }
    return rett;
  }

  /**
  * write the CMrequest to the output stream 'out'. 
  * @param out: an OutputStream to which the representation for the CMrequest
  * 		will be written. 
  * @return true if the CMrequest was written, false if some error occured. 
  */
  public boolean toStream(OutputStream out) {
    boolean done;

    try {
      DataOutputStream dout = new DataOutputStream(out);

      // 1: command, an integer
      dout.writeInt(cmd);
      
      // 2: possible ID of from port
      if(from == null)
	dout.writeBoolean(false);
      else {
	dout.writeBoolean(true);
	from.toStream(dout);
      }
      
      // 3: possible ID of to port
      if(to == null)
	dout.writeBoolean(false);
      else {
	dout.writeBoolean(true);
	to.toStream(dout);
      }
      
      // 4: posible String destination
      if(dest == null)
	dout.writeBoolean(false);
      else {
	dout.writeBoolean(true);
	dout.writeInt(dest.length());
	dout.writeBytes(dest);
      }
      
      // 5: possible String caller address
      if(caller == null)
	dout.writeBoolean(false);
      else {
	dout.writeBoolean(true);
	dout.writeInt(caller.length());
	dout.writeBytes(caller);
      }
      
      // 6: destination port, integer
      dout.writeInt(portDest);
      // 7: caller port, integer
      dout.writeInt(portCaller);

      dout.flush();
      done = true;
    }
    catch(IOException ioe)
      {
	done = false;
      }

    return done;
  }

  /**
   * test stub ONLY. Only tries to make three IDs and compare them! 
   */
  public static void main(String [] argv)
  {
    ID from = new ID("blah1", 1);
    ID to = new ID("blah2", 2);
    CMrequest req = new CMrequest(CMrequest.ESTABLISH, from, to, "callerHost", "destHost",  1234, 15);
    //    CMrequest req = new CMrequest();
    if(argv[0].equals("get")) 
      {
	//System.out.println("cmd: " + req.getCommand());
//	System.out.println("from: ");
	req.getFromPort().toStream(System.out);
//	System.out.println("to: ");
	req.getToPort().toStream(System.out);
//	System.out.println("dest: " + req.getToHost());
//	System.out.println("caller: " + req.getCallerHost());
//	System.out.println("port: " + req.getCallerPort());
      }
    else if(argv[0].equals("from"))
      {
	CMrequest received = new CMrequest();
	if(!received.fromStream(System.in))
	  System.out.println("error occured!");
	else
	  received.toStream(System.out);
      }
    else if(argv[0].equals("to"))
      req.toStream(System.out);
  }
}

