package webflow.backend;

import java.io.*;
import java.util.*;

/**
* the MMreply class is used when sending requests to the ModuleManager 
* class. It extends the Message class. It stores a fewitems in it: 
* - the status (one of an enumeration: OK or ERROR)
* - a MetaModule object (when an module is created)
* - a description (if an error occured)
* For the description of the communication protocol which MMreply
* implements look under communication protocols. 
*/
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class MMreply extends Message {
  /** 
  * an enumeration of available statuses that the ModuleManager might 
  * return 
  */
  public static final int ERROR = 0, OK = 1;
  // MMreply carries a status, a metamodule, and an error description string
  int stat;
  MetaModule mmeta;
  String des;
  
  /** 
  * void constructor used by the receiving side to create an empty 
  * MMreply object to get the incomming message
  */
  public
  MMreply() 
  {
    stat=0;
    mmeta = null;
    des = null;
  }

 
  /** 
  * constructor used to implement replies to RUN, STOP, and DESTROY
  * requests (that need only a status)
  * @param status - return status
  */
  MMreply(int status) {
    stat=status;
    mmeta = null;
    des = null;
  }

  /** 
  * constructor used to reply when an error occured
  * @param status - return status
  * @param description - description of error
  */
  MMreply(int status, String description) {
    stat = status;
    des = description;
    mmeta = null;
  }

  /**
  * constructor used to reply to the INITIALIZE request
  * @param status - return status
  * @param meta - MetaModule containing module data
  */
  MMreply(int status, MetaModule meta) {
    stat=status;
    mmeta=meta;
    des = null;
  }

  /** 
  * return the status stored in the object
  * @return int - status stored in the reply object
  */
  public int getStatus() {
    return stat;
  }

  /**
  * return the MetaModule stored in the object
  * @return MetaModule - the MetaModule stored in the reply object
  */
  public MetaModule getMetaModule() {
    return mmeta;
  }

  /**
  * return the error description stored in the object
  * @return String - the String stored in the reply object
  */
  public String getDescription() {
    return des;
  }

  /**
  * read the MMreply from the input stream 'in'. 
  * @param in: an InputStream from which the representation for the 
  * 	       MMreply will be read. 
  * @return true if the MMreply was read, false if some error occured. 
  */
  public boolean fromStream(InputStream in) 
  {
   boolean done=true;
   
   try
   {
     DataInputStream din = new DataInputStream(in);
     // 1: int - the return status
     stat = din.readInt();

     // 2: possible MetaModule
     if (din.readBoolean())
     {
       mmeta = new MetaModule();
       mmeta.fromStream(din);
     }

     // 3: possible error description string
     if (din.readBoolean())
     {
       int bufsize = din.readInt();
       byte[] b = new byte[bufsize];
       din.readFully(b,0,bufsize);
       des = new String(b,0,0,bufsize); 
     }
     else
       des = null;
     done =true;
   }
   catch (IOException e )
     {
       mmeta = null;
       des = null;
       done = false;
       stat = 0;
     }
   return done;
  }

  /**
  * write the MMreply to the output stream 'out'. 
  * @param out: an OutputStream to which the representation for the MMreply 
  * 		will be written. 
  * @return true if the MMreply was written, false if some error occured. 
  */
  public boolean toStream(OutputStream out) 
  {
    boolean done;
    try
      {
	DataOutputStream  dout = new DataOutputStream(out);
	// 1: int - the return status
	dout.writeInt(stat);
	
	// 2: possible MetaModule
	if (mmeta == null)
	  dout.writeBoolean(false);
	else
	  {
	    dout.writeBoolean(true);
	    mmeta.toStream(dout);
	  }

	// 3: possible error description string
	if (des==null)
	  dout.writeBoolean(false);
	else
	  {
	    dout.writeBoolean(true);
	    dout.writeInt(des.length());
	    dout.writeBytes(des);
	  }
	dout.flush();
	done =true;
      }
    catch (IOException e)
      { done =false;}
    return done;
  }
  

  /* Test stub */
  public static void main(String[] argv)
  {
    MetaModule mmeta = new MetaModule("Addition of numbers");
    MMreply req = new MMreply(MMreply.OK,mmeta);
    MMreply req1 = new MMreply(MMreply.ERROR, "Error message is: Dumb Error");
    if (argv[0].equals("give"))
      {
	System.out.println("stat:" + req.getStatus());
	req.getMetaModule().toStream(System.out);
      }
    else if (argv[0].equals("from"))
      {
        MMreply reply = new MMreply();
	if(!reply.fromStream(System.in))
	  System.out.println("error  occured");
	else
	  reply.toStream(System.out);	  
      }
    else   if(argv[0].equals("to1"))
      req.toStream(System.out);
    else if(argv[0].equals("to2"))
      req1.toStream(System.out);
  }
}

