package webflow.backend;

import java.io.*;
import java.util.Vector;

/**
* the MMrequest class is used when sending requests to the ModuleManager 
* class. It extends the Message class. It stores three items in it: 
* - the command (one of an enumeration: INITIALIZE, RUN, STOP, or DESTROY
* - an ID: used with RUN, STOP, and DESTROY to identify the module to 
*   whom the command pertains
* - a string used with INITIALIZE to identify (by name) which module to 
*   create
* For the description of the communication protocol which MMrequest 
* implements look under communication protocols. 
*/
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class MMrequest extends Message {
  /** 
   * an enumeration of available commands to be sent through a MMrequest
   */
  public static final int INITIALIZE = 0, RUN = 1, STOP = 2, DESTROY = 3, RUN_METHOD = 4;
  
  // MMrequest carries (at most) a command, a module ID and a name of a module
  int cmd;
  ID mmid;
  String name;
  String methodName;
  Vector params;
  
  /** 
  * void constructor used by the receiving side to create an empty 
  * MMrequest object to get the incomming message
  */
  MMrequest() {
    cmd=0;
    mmid = null;
    methodName = null;
    params = null;
  }

  /** 
  * constructor used to implement the RUN_METHOD
  * commands (that need a command and module ID)
  * @param command - int: command requested
  * @param mID - ID: module ID to which the command pertains
  * @param 
  */
  public
  MMrequest(int command, ID mID, String _methodName, Vector _params) 
  {
    cmd = command;
    mmid = mID;
    name = mID.getModuleName();
    methodName = _methodName;
    params = _params;
  }// end of constructor
 

  /** 
  * constructor used to implement the RUN, STOP, and DESTROY 
  * commands (that need a command and module ID)
  * @param command - int: command requested
  * @param mID - ID: module ID to which the command pertains
  */
  public
  MMrequest(int command, ID mID) {
    cmd = command;
    mmid = mID;
    name = null;
    methodName = null;
    params = null;
  }

  /**
  * constructor used to implement the INITIALIZE command
  * @param command - int: command requested
  * @param which - String: which module to create
  */
  public
  MMrequest(int command, String which) {
    cmd = command;
    name =which;
    mmid = null;
    methodName = null;
    params = null;
  }

  /**
  * return the command stored in MMrequest
  * @return value of the command stored in MMrequest
  */
  public int getCommand() {
    return  cmd;
  }

  /**
  * return the moduleID parameter stored in MMrequest 
  * @return ID: value of the moduleID stored in the object
  */
  public ID getModuleID() {
    return mmid;
  }

  /**
  * return the 'moduleName' parameter stored in MMrequest
  * @return String stored in the 'moduleName' parameter
  */
  public String getModuleName() {
    return name;
  }

  /**
  * read the MMrequest from the input stream 'in'. 
  * @param in: an InputStream from which the representation for the 
  * 	       MMrequest will be read. 
  * @return true if the MMrequest was read, false if some error occured. 
  */
  public boolean fromStream(InputStream in) {
    boolean done ;
    try
      {
	DataInputStream din = new DataInputStream(in);
	// 1: int - the command
	cmd=din.readInt();

	// 2: possible module ID
	if (din.readBoolean())
	  {
	    mmid = new ID();
	    mmid.fromStream(din);
	  }
	else
	  mmid = null;

	// 3: possible module name - a String
	if (din.readBoolean())
	  {
	    int bufsize = din.readInt();
	    byte[] b = new byte[bufsize];
	    din.readFully(b,0,bufsize);
	    name = new String(b,0,0,bufsize);
	  }
	else name=null;
	
	done =true;
      }
    
    
    catch(IOException e) {
      done =false;
      mmid= null;
    }
    
    return done;
  }
  
  /**
   * write the MMrequest to the output stream 'out'. 
   * @param out: an OutputStream to which the representation for the MMrequest
   * 		will be written. 
   * @return true if the MMrequest was written, false if some error occured. 
   */
  public boolean 
  toStream(OutputStream out) 
  { 
    boolean done;
    try
    {  
       DataOutputStream dout = new DataOutputStream(out);
       //ObjectOutputStream dout = new ObjectOutputStream(out);

	   // 1: int - the command
	   dout.writeInt(cmd);

	   // 2: possible module ID
	   if (mmid==null)
	      dout.writeBoolean(false);
	   else 
	    { dout.writeBoolean(true);
	      mmid.toStream(dout);
	    }

	   // 3: possible module name - a String
	   if (name==null)
	       dout.writeBoolean(false);
	   else
	    {
	       dout.writeBoolean(true);
	       dout.writeInt(name.length());
	       dout.writeBytes(name);
	    }

       // 4: possible method name - a String
       if(methodName == null )
          dout.writeBoolean(false);
       else
        { 
          dout.writeBoolean(true); 
          dout.writeInt(methodName.length());
          dout.writeBytes(methodName);
        }

       // 5: possible parameters - a Vector
       if(params == null )
          dout.writeBoolean(false);
       else
        {
          //params.writeObject(dout);
        } 

	   dout.flush();
	   done=true;

    } catch(IOException e)
      { done=false;
	    System.out.println("exception" + e);
      }

    return done;

  }

  /* test stub */
  public static void main(String[] argv)
  {
    ID mmid = new ID("blah1",1);
    MMrequest req = new MMrequest(MMrequest.INITIALIZE,"Module#1");
    MMrequest req1 = new MMrequest(MMrequest.RUN,mmid);
    if(argv[0].equals("get"))
      {
	System.out.println("cmd:" + req1.getCommand());
	System.out.println("which" + req1.getModuleID());
	System.out.println("cmd" + req.getCommand());
      }
    else if(argv[0].equals("from"))
      {
	if(!req1.fromStream(System.in))
          System.out.println("error  occured");
	else
	  {
	    System.out.println("cmd " + req.getCommand());
	    ID mID = req1.getModuleID();
	    System.out.println("old mID == new mID? " + mmid.equals(mID));
	  }
      }
    else if(argv[0].equals("to1"))
      req1.toStream(System.out);
    else if(argv[0].equals("to2"))
      req.toStream(System.out);
  }
}

