
package webflow.backend;

import java.io.*;
import java.util.*;

/**
* the MetaModule class will return information from the module to the 
* ModuleManager, and ultimately to the SessionManager. It stores a 
* moduleID, and a list of portIDs. It also stores a description of the 
* Module (for now, only a string). 
* The class provides methods for access to each of the elements it stores
*/
public class 
MetaModule extends Object
{

  /** MetaModule state:  */
  String description; 

  /** ID of this module */
  ID moduleID;
 
  /** Port List of this module */
  Vector portStorage;

  /** HTML representation of this module */
  String htmlString;

  /** Name of the Netscape Window */
  String viewerWindowName = null;
  
  /** current port. Used by the iterator: getFirstPort,  */
  /** and getNextPort.                                   */ 
  /** stores the last element of portStorage accessed    */
  private int current; 

  /**
  * void constructor used by when communicating a MetaModule over a
  * stream. The receiving side creates an empty MetaModule, then
  * receives it from a stream.
  */
  MetaModule() {
    description = null;
    moduleID = null;
    portStorage = null;
    htmlString = null;
  }

  /** 
  * creator of the MetaModule class. MetaModules are created with a 
  * description. 
  * Inputs: 
  * @param d: String containing the description of the module
  */
  public MetaModule(String d) {
    description = d;
    moduleID = null;
    portStorage = new Vector();
    htmlString = null;
  }
 
  /**
  * set the moduleID inside the MetaModule object
  * @param mID: ID containing the module's unique ID
  * @return nothing
  */
  public void setModuleID(ID mID) {
    moduleID = mID;
  }

  /**
  * return the module ID stored in the MetaModule object
  * @return ID containing the module ID
  */
  public ID getModuleID() {
    return moduleID;
  }

  /**
  * place the port's ID in the list of port IDs inside the MetaModule 
  * object 
  * @param pID: ID containing the portID to be stored in the MeatModule 
  * 	   object 
  * @return nothing
  */
  public void putPortID(ID pID) {
    portStorage.addElement(pID);
  }

  /**
  * get the first port ID from the MetaModule object
  * @return ID of the first port in the MetaModule, or null if none
  */
  public ID getFirstPortID() {
    if(portStorage.size() == 0)
      return null;
    current = 1;
    ID toReturn;
    try
      {
	toReturn = (ID) portStorage.elementAt(0);
      }
    catch(Exception e)
      { toReturn = null; }

    return toReturn;
  }

  /**
  * get the next port ID from the MetaModule object
  * this method forms an iterator with the getFirstPortID() method. 
  * @return ID of the next port in the MetaModule, or null if none left 
  */
  public ID getNextPortID() {
    if(portStorage.size() <= current)
      return null;
    current++;
    ID toReturn;
    try
      {
	toReturn = (ID) portStorage.elementAt(current-1);
      }
    catch(Exception e)
      { toReturn = null; }

    return toReturn;
  }

  /** 
  * return the description stored in the MetaModule object
  * @return String containing the description
  */
  public String getDescription() {
    return description;
  }

  /**
   * set the htmlString in the MetaModule object
   * @param html - html string to be stored in the MetaModule object
   */
  public void setHTML(String html) {
    htmlString = html;
  }

  /**
   * return the htmlString stored in the MetaModule object * @return String containing the htmlString stored in the MetaModule object
   */
  public String getHTML() {
    return(htmlString);
  }

  /**
   * read the MetaModule object from the input stream 'in'. 
   * @param in: an InputStream from which the representation for the
   *            object will be read. 
   * @return true if the object was read, false if some error occured. 
   */
  public boolean fromStream(InputStream in) {
    boolean rett;

    try
      {
	DataInputStream din = new DataInputStream(in);
	// 1: possible String - module description
  	if (din.readBoolean())
        {
	  int bufsize = din.readInt();
	  byte[] b = new byte[bufsize];
	  din.readFully(b,0,bufsize);
	  description = new String(b,0,0,bufsize);
	}

	// 2: possible String - html string for the viewer modules
  	if (din.readBoolean())
        {
	  int bufsize = din.readInt();
	  byte[] b = new byte[bufsize];
	  din.readFully(b,0,bufsize);
	  htmlString = new String(b,0,0,bufsize);
	}

	// 3: possible ID - module ID
	if(din.readBoolean())
	{
	  moduleID = new ID();
          moduleID.fromStream(din);
	}

	// 4: possible int - size, followed by a Vector - list of port IDs
	if(din.readBoolean())
	{
	  int size = din.readInt();
	  portStorage = new Vector(size);
	  int i;
	  for(i=0; i<size; i++)
	    {
	      ID portID = new ID();
	      portID.fromStream(din);
	      putPortID(portID);
	    }
	}
	        
	rett = true;
      }
    catch(IOException e)
      {
	rett = false;
	moduleID = null;
	description = null;
	htmlString = null;
      }
    return rett;
  }

  /**
   * write the MetaModule to the output stream 'out'. 
   * @param out: an OutputStream to which the representation of the 
   *             object will be written. 
   * @return true if the object was written, false if some error occured. 
   */
  public boolean toStream(OutputStream out) {
    boolean done;
    try{
      DataOutputStream dout = new DataOutputStream(out);
      
      // 1: possible String - module description
      if(description == null)
	dout.writeBoolean(false);
      else{
	dout.writeBoolean(true);
	dout.writeInt(description.length());
	dout.writeBytes(description);
      }
      
      // 2: possible String - html string for the viewer modules
      if(htmlString == null)
	dout.writeBoolean(false);
      else{
	dout.writeBoolean(true);
	dout.writeInt(htmlString.length());
	dout.writeBytes(htmlString);
      }
      
      // 3: possible ID - module ID
      if(moduleID == null)
	dout.writeBoolean(false);
      else
	{
	  dout.writeBoolean(true);
	  moduleID.toStream(dout);
	}
      
      // 4: possible int - size, followed by a Vector - list of port IDs
      if(portStorage == null)
	dout.writeBoolean(false);
      else
	{
	  dout.writeBoolean(true);
	  dout.writeInt(portStorage.size());
	  int i;
	  for(i=0; i<portStorage.size(); i++)
	    {
	      ID portID = (ID) portStorage.elementAt(i);
	      portID.toStream(dout);
	    }
	}
      
      dout.flush();
      done=true;
    }
    
    catch (IOException e) {
      done =false;
    }
    return done;
  }

  /**
  * Sets the name of the window 
  * @param _WinName Name of the Window
  */
  public void
  setViewerWinName(String _WinName)
  {  viewerWindowName = new String(_WinName); }

  /** 
  * Returns the name of window
  * @return the Name of window
  */
  public String
  getViewerWinName()
  { return (viewerWindowName!=null? new String(viewerWindowName): null); }


  /**
   * test stub ONLY.
   */
  public static void main(String [] argv)
  {
    MetaModule mm = new MetaModule("describe!");
    ID one = new ID("blah1", 1);
    ID two = new ID("blah2", 2);
    ID mID = new ID("ModuleID", 123);
    mm.putPortID(one);
    mm.putPortID(two);
    mm.setModuleID(mID);
    mm.setHTML("Dumb HTML");

    if(argv[0].equals("get"))
      {
	ID blah;
	blah = mm.getFirstPortID();
	while(blah != null)
	  {
	    blah.toStream(System.out);
	    blah = mm.getNextPortID();
	  }
	mm.getModuleID().toStream(System.out);
	System.out.println("Description: " + mm.getDescription());
      }
    else if(argv[0].equals("from"))
      {
	MetaModule mm1 = new MetaModule();
	if(mm1.fromStream(System.in))
	  {
	    ID blah;
	    blah = mm.getFirstPortID();
	    while(blah != null)
	      {
		blah.toStream(System.out);
		blah = mm.getNextPortID();
	      }
	    ID modID = mm.getModuleID();
	    if(modID != null)
	      modID.toStream(System.out);
	    System.out.println("Description: " + mm.getDescription());
	    System.out.println("HTML: " + mm.getHTML());
	  }
      }
    else if(argv[0].equals("to"))
      mm.toStream(System.out);
  }
}
