package webflow.backend;

import java.util.*;
import java.net.*;
import javax.servlet.*;
import java.io.*;

/**
 * ModuleManager is a jeeves Servlet. It maintains a list of all modules under
 * one server. It can  initialize module, and execute it.
 */
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class 
ModuleManager extends GenericServlet 
              implements Runnable, WebFlowDebugFlags 
{
  /** Hashtable of ModuleWrappers, indexed by the IDs of the modules they wrap */
  Hashtable moduleList; 

  /** Server socket on which the ModuleManager will listen for incoming requests */
  ServerSocket listener;

  /**
  * host name and port number on which the ModuleManager's server socket is
  * listening. 
  */
  String hostName;
  int portNumber;

  /** thread which will service the listener socket */
  Thread socketThread;

  /** factory to make IDs */
  IDFactory factory;

  /** diagnostic object */
  Diagnostic diag;

  /** diagnostics on or off (true = on, false = off) */
  static final boolean DIAGNOSTICS_ACTIVE = true;

  /**
   * initialize the ModuleManager. It initializes the list of the modules.  
   */
  public void init(ServletConfig Params)
  {
    System.out.println("Module Manager accessed");
    try { 
          super.init(Params);
    } catch (ServletException e)
      {  System.out.println(" Exception in init() of MM ");
         e.printStackTrace();
      }
    moduleList = new Hashtable();
    listener = null;
    portNumber = 0;
    socketThread = null;
    System.out.println("Module Manager accessed (2)");
    // initialize the diagnostic object
    diag = new Diagnostic("mm.diag");
    diag.activate(DIAGNOSTICS_ACTIVE);
    diag.write("Activated diagnostics object");
    System.out.println("Activated diagnostics object");

    try 
      {
	// get local host name
	InetAddress local = java.net.InetAddress.getLocalHost();
	hostName = local.getHostName();

	// open new server socket
	listener = new ServerSocket(0);
	portNumber = listener.getLocalPort();

	// create the IDFactory
	factory = new IDFactory(hostName+":"+portNumber+":ModuleManager");

	// finally, branch new thread, set it up as a deamon (to terminate
	// upon completion of master program), and start it up
	socketThread = new Thread(this);
	socketThread.setDaemon(true);
	socketThread.start();
      }
    // for now: nothing I can do with exceptions. 
    // Leave things as they are initialized (to null values)
    catch(UnknownHostException e)
    {  hostName = "0"; 
       //System.out.println(" UnknownHostException in init() of MM ");
       e.printStackTrace();
    }
    catch(IOException e)
    {  hostName = "0";
       System.out.println(" UnknownHostException in init() of MM ");
       e.printStackTrace();
    }
  }


  /**
   * service method is called when the servlet is refered by URL. It defines
   * the response of the servlet, wich is the address of the socket on wich 
   * this servlet will communicate with the rest of the world.
   * @param req: a ServletRequest wich contains the parametars of the request
   * @param res: a ServletResponse, an object that contains the response of the
   *             servlet.
   */

  public void service(ServletRequest req, ServletResponse res) 
       throws ServletException, IOException
  {
    String command = req.getParameter("cmd");
    if(command!=null && command.equals("get_running_modules"))
    {   res.setContentType("text/plain");
        OutputStream os = res.getOutputStream();
        PrintStream out = new PrintStream( os );
        
        Enumeration runningModules = moduleList.elements();

        for(; runningModules.hasMoreElements();)
        { ModuleWrapper tmpMW = (ModuleWrapper) runningModules.nextElement();
          out.println(tmpMW.getModuleName()+" "+tmpMW.getModuleStatus()); 
        }// end of for()

        out.flush();
    }
    else
    {   // one day there will be multiple requests. For now, the only request
        // is get address of ModuleManager's socket. 
        res.setContentType("text/plain");
        OutputStream os = res.getOutputStream();
        PrintStream out = new PrintStream( os );
        out.println(hostName);
        out.println(portNumber);
        out.println("TIMUCIN");
        out.flush();
    }
  }// end of service()


  /** 
   * Provides an information about the servlet.
   * @return String: description of the servlet.
   */
  public String getServletInfo(){
    return "Module Manager servlet";
  }

  /**
   * listen on socket listener, accept MMrequests, process them, 
   * and return MMreply objects. 
   */
  public void run() {
    boolean finished = false;
    // notice that this loop is in fact an endless loop. It could be made
    // to finish if necessary. 
    while(!finished)
      {
	try 
	  {
	    Socket aSocket = listener.accept();

	    diag.write("received request");

	    MMrequest request = new MMrequest();
	    request.fromStream(aSocket.getInputStream());
	    MMreply reply = serveRequest(request);

	    diag.write("composed reply with status " + reply.getStatus());

	    if(reply != null)
	      reply.toStream(aSocket.getOutputStream());
	    aSocket.close();
	    diag.write("sent reply, closing connection");
	  }
	// for now do nothing with the exception. Just retry locally
	catch(IOException ioe)
	  {}
      }
  }

  /**
   * serve request to initialize a module IDd by mID
   * @param mName : module name - which module to initialize
   * @return MMreply containing the reply information. 
   */
  MMreply initializeModule(String mName) {
    MMreply rett = null;

    diag.write("will initialize module " + mName);

    ModuleWrapper mw = new ModuleWrapper();

    MetaModule mm = mw.initializeModule(mName);
    diag.write("constructed meta module for " + mName);
    if(mm == null)
      rett = new MMreply(MMreply.ERROR, "Could not initialize " + mName);
    else
      { 
	diag.write("meta module is OK");
	// assign a unique ID to the module
	mm.setModuleID(factory.getUniqueID());

	// reply OK and insert the modulewrapper in the module list
	rett = new MMreply(MMreply.OK, mm);
	diag.write("constructed an OK reply");
	moduleList.put(mm.getModuleID(), mw);
	diag.write("module added in module list");
      }
    return rett;
  }

  /**
   * serve request to run a module IDd by mID
   * @param mID : module ID - which module to run
   * @return MMreply containing the reply information. 
   */
  MMreply runModule(ID mID) {
    MMreply rett = null;

    // get the ModuleWrapper that wraps mID
    ModuleWrapper mw = (ModuleWrapper)moduleList.get(mID);
    if( mw == null )
	rett = new MMreply(MMreply.ERROR, "Module not found");
    else
      {
	mw.runModule();
	rett = new MMreply(MMreply.OK);
      }

    return rett;
  }
  
  /**
   * serve request to stop a module IDd by mID
   * @param mID : module ID - which module to stop
   * @return MMreply containing the reply information. 
   */
  MMreply stopModule(ID mID){
    MMreply rett = null;

    // get the ModuleWrapper that wraps mID
    ModuleWrapper mw = (ModuleWrapper)moduleList.get(mID);
    if( mw == null )
	rett = new MMreply(MMreply.ERROR, "Module not found");
    else
      {
	mw.stopModule();
	rett = new MMreply(MMreply.OK);
      }
    return rett;
  }

  /**
   * serve request to destroy a module IDd by mID
   * @param mID : module ID - which module to destroy
   * @return MMreply containing the reply information. 
   */
  MMreply destroyModule(ID mID) 
  {
    MMreply rett = null;

    if(DEBUG==true) 
       System.out.println(" In destroyModule(ID mID) ");

    /** get the ModuleWrapper that wraps mID */
    ModuleWrapper mw = (ModuleWrapper)moduleList.get(mID);

    if( mw == null )
       rett = new MMreply(MMreply.ERROR, "Module not found");
    else
    {  mw.destroyModule();
       rett = new MMreply(MMreply.OK);
       
       /** removes the module from the hashtable.  */
       moduleList.remove(mID);

    }

    return rett;

  }// end of destroyModule()

  /**
   * serve the request. Just a giant switch statement
   * @param req: MMrequest being served
   * @return MMreply to the MMrequest. 
   */
  MMreply serveRequest(MMrequest req) {
    MMreply reply = null;
    int command = req.getCommand();

    diag.write("request has command " + command);
    // call the apropriate service method, depending on the command
    // in the request
    switch(command) {
    case MMrequest.INITIALIZE: 
      reply = initializeModule(req.getModuleName());
      break;
    case MMrequest.RUN: 
      reply = runModule(req.getModuleID());
      break;
    case MMrequest.STOP: 
      reply = stopModule(req.getModuleID());
      break;
    case MMrequest.DESTROY: 
      reply = destroyModule(req.getModuleID());
      break;
    default: 
      reply = new MMreply(MMreply.ERROR, "Command not recognized");
      break;
    }
    diag.write("served request with command " + command);
    return reply;
  }

  /* test stub. Will not work unless serveRequest is made public
     commented out, after the first part of unit testing stage
  public static void main(String args[])
  {
    ModuleManager mm = new ModuleManager();
    mm.init();
    MMrequest req = new MMrequest(MMrequest.INITIALIZE, "Trying");
    MMreply reply = mm.serveRequest(req);
    //System.out.println("reply to init is: ");
    if(reply != null)
      reply.toStream(System.out);
    else
      System.out.println("reply is null");

    ID mID = reply.getMetaModule().getModuleID();
    req = new MMrequest(MMrequest.RUN, mID);
    reply = mm.serveRequest(req);
    //System.out.println("reply to run is: ");
    if(reply != null)
      reply.toStream(System.out);
    else
      System.out.println("reply is null");
  }
  */  
}


