
package webflow.backend;

import java.net.*;
import java.io.*;

/**
* the ModuleWrapper class will provide a wrapper for the modules to be
* placed inside the ModuleManager. The ModuleManager will send commands 
* to the ModuleWrapper, which will forward them to the Module itself. 
* The advantage is that the ModuleWrapper can return control to the 
* ModuleManager instantly, especially when running the Module. No need
* to bother the user with such details. 
*/
public class 
ModuleWrapper implements Runnable, WebFlowDebugFlags 
{
  /** */
  Thread moduleWrapperThread;

  /** */
  Module wrappedModule;

  /** diagnostic object */
  Diagnostic diag;

  /** diagnostics on or off (true = on, false = off) */
  static final boolean DIAGNOSTICS_ACTIVE = false;

  /** */
  String moduleName = null;

  /** */ 
  public static final int NOT_INITIALIZED= -1;
  public static final int INITIALIZED    = 1;
  public static final int RUNNING        = 2;
  public static final int STOPPED        = 3;
  public static final int DESTROYED      = 4;

  /** */
  int status = -1;

  /** 
  * creator for ModuleWrapper. 
  */
  public
  ModuleWrapper() 
  {
    /** There is no module binding and thread for this */
    moduleWrapperThread = null;
    wrappedModule = null;

    /** initialize the diagnostic object */
    diag = new Diagnostic("mw.diag");
    diag.activate(DIAGNOSTICS_ACTIVE);
    diag.write("Activated diagnostics object");
    if(DEBUG==true)
      System.out.println("Activated diagnostics object");

  }// end of constructor

  /**
  * Initializes the ModuleWrapper by loading the class whichModule.
  * Directly calls the whichModule's initialize class
  * 
  * @param whichModule - String giving the name of the module to 
  *     		 load (i.e. initialize)
  * @return - either the MetaModule returned by the initialization of
  * 	      whichModule, or null if an error occured
  */
  public MetaModule 
  initializeModule(String whichModule) 
  {  MetaModule rett = null;

     diag.write("received reqeust to initialize module " + whichModule);

     if(DEBUG == true)
       System.out.println("received reqeust to initialize module " + whichModule);

    /** try loading a new class, getting its instance, and initializing it.  */
    try
    {  
       /** Load the class */
       Class moduleClass = Class.forName(whichModule);
       
       /** create an instance of this class */
	   wrappedModule = (Module) moduleClass.newInstance();

	   diag.write("created new instance");

       /** 
       * use Modul interface and run the 
       * initialize method of  the wrapped module
       */
	   rett = wrappedModule.initialize();
	   diag.write("initialized it");
	   if(DEBUG == true) System.out.println("initialized it");
       moduleName = whichModule;

       /** Update the status of the module */
       status = INITIALIZED;

    } /** so far, no action taken on exceptions.  */
      /** The method will just return null        */
      catch(ClassNotFoundException ce) 
      {  diag.write("class not found problem");
         if(DEBUG == true) System.out.println("class not found problem");
         diag.write("exception " + ce);
         if(DEBUG == true) System.out.println("exception " + ce);
         rett = null;
         ce.printStackTrace();
      }
      catch(InstantiationException ie) 
      {
         diag.write("instantiation problem");
         if(DEBUG == true) System.out.println("instantiation problem");
         diag.write("exception " + ie);
         if(DEBUG == true) System.out.println("exception " + ie);
         rett = null;
         ie.printStackTrace();
      }
      catch(IllegalAccessException iae) 
      {
         diag.write("illegal access problem");
         if(DEBUG == true) System.out.println("illegal access problem");
         diag.write("exception " + iae);
         if(DEBUG == true) System.out.println("exception " + iae);
         rett = null;
         iae.printStackTrace();
      }

    return rett;

  }// end of initializeModule()
 
  /**
  * runs the module code. Control is immediately returned to the 
  * ModuleManager, and the module is ran in a separate thread. 
  */ 
  public void 
  runModule() 
  {
    if(moduleWrapperThread == null)
    {  
       moduleWrapperThread = new Thread(this);
       moduleWrapperThread.start();

    }// end of if

  }// end of runModule()

  /** 
  * stops the execution of the module. 
  */
  public void 
  stopModule() 
  {
    /** if the thread is still running, first stop it, */
    if(moduleWrapperThread != null)
    {  
       moduleWrapperThread.stop();

       /** throw away the reference to the thread, */ 
       /** so that it can be reused.               */
       moduleWrapperThread = null;

    }// end of if

    /** now stop can be safely called (no thread collisions) */
    wrappedModule.stop();

    /** update the status of the module */
    status = STOPPED;

  }// end of stopModule()

  /**
  * destroys the module. 
  */
  public void 
  destroyModule() 
  {
    if(DEBUG==true)
       System.out.println(" IN destroyModule() of ModuleWrapper ");

    /** if the thread is still running, first stop it, */
    if(moduleWrapperThread != null)
    {
       if(DEBUG==true)
          System.out.println(" Thread is exist!");

       moduleWrapperThread.stop();

       if(DEBUG==true)
          System.out.println(" After Thread.stop() ");

	   /** throw away the reference to the thread, */
       /** so that it can be reused.               */
	   moduleWrapperThread = null;

       if(DEBUG==true)
          System.out.println(" After Thread = null ");

    }// end of if
    else if(DEBUG==true)
          System.out.println(" Thread is not exist! ");
    
    /**
    if(status == INITIALIZED && wrappedModule.getSocket()!=null)
    {
      try
      {  Socket aSocket = wrappedModule.getSocket().accept();

         DataOutputStream dos =new DataOutputStream(aSocket.getOutputStream());
       
         dos.writeInt(1);
         dos.close();
         aSocket.close();
 
      } catch(IOException ei)
        {  System.out.println(" IOException is raised !");
           ei.printStackTrace();
        }
    
      try
      {  wrappedModule.getSocket().close();
      } catch(IOException ei)
        {  System.out.println(" IOException is raised !");
           ei.printStackTrace();
        }

    }
    */

    /** now destroy can be safely called (no thread collisions) */
    wrappedModule.destroy();

    /** update the status of the module */
    status = DESTROYED;

  }// end of destroyModule()

  /**
  * runs the wrapped module's run method. 
  */
  public void 
  run() 
  {
    if(DEBUG==true) System.out.println("RUNNING!!!");
    wrappedModule.run();

    /** update the status of the module */
    status = RUNNING;

  }// end of run()

  /**
  * returns the hooked module name.
  */
  public String 
  getModuleName() 
  { return new String(moduleName); }

  /**
  * retuns the status of the module.
  */
  public String 
  getModuleStatus()
  {  String[] st = { "INITIALIZED","RUNNING","STOPPED","DESTROYED"};
     String ret  = new String("NOT_INITIALIZED");
     
     if(status>0 && status<5) ret = new String(st[status-1]);
    
     return ret;

  }// end of getModuleStatus() 

  // Test bed. 
  public static void main(String args[])
  {
    ModuleWrapper mw = new ModuleWrapper();

    MetaModule mm = mw.initializeModule("Trying");
    if(mm != null)
      mm.toStream(System.out);
    mw.runModule();
    // some code to pause here!
    System.out.println("Wait a while, then press enter");
    try {
      int x = System.in.read();
    }
    catch(Exception e) {}
    mw.stopModule();
    mw.destroyModule();
  }

}// end of class ModulWrapper

