package webflow.backend;

import java.net.*;
import java.io.*;

/**
* the PortRepresentation class will be used by the ConnectionManager to
* store and access data connected to the ports it services. As a minimum, 
* the PortRepresentation will store the port description, port reference, 
* is the port connected. 
*/
public class PortRepresentation implements Runnable {
  // reference to the port stored in the representation. 
  Port storedPort;

  // description of the stored port
  String description; 

  // is the port already connected or not
  boolean connected; 

  // variables used only when connecting to a server socket. Thread that will 
  // run in 'background', and name of host/number of port to connect to. 
  Thread connectPort;
  String hostName;
  int portNumber;

  /** 
  * creator for PortRepresentation. 
  * @param whichPort: reference to a Port that is stored in this 
  *		      representation. 
  * @param desc: String which gives the Port description. 
  */
  public PortRepresentation(Port whichPort, String desc) {
    storedPort = whichPort;
    description = desc;
    connected = false;
  }

  /**
  * returns the description of the stored port. 
  * @return String giving the description of the stored port. 
  */
  public String getDescription() {
    return description;
  }

  /**
  * is the represented port already connected or not. 
  * @return boolean : is the port already connected?
  */
  public boolean getConnected() {
    return connected;
  }

  /**
  * give a socket to the port represented by this PortRepresentation
  * @param s: Socket on which the Port will listen/send
  */
  public void giveSocket(Socket s) {
    storedPort.receiveConnection(s);
    connected = true;
  }

  /**
   * Create a separate thread that will make a socket, connect it to 
   * the server socket at hostName:portNumber, and pass it to the port. 
   * @param hName: String - name of host to which to connect
   * @param pNumber: int - port number on the host
   */
  public void establishConnection(String hName, int pNumber) {
    hostName = hName;
    portNumber = pNumber;
    connectPort = new Thread(this);
    connectPort.start();
  }

  /**
   * connect to the server socket listening for connection, and pass the 
   * socket to the port. 
   */
  public void run() {
    try {
      Socket s = new Socket(hostName, portNumber);
      giveSocket(s);
    }
    catch(UnknownHostException uhe) {
    }
    catch(IOException ioe) {
    }
  }

  /* Test stub 
     Must be commented out, since it uses the class NullPort - which does not
     exist in the release (testing purposes only!)
  public static void main(String args[]) {
    ID pID = new ID("Blah", 1);
    NullPort np = new NullPort();
    PortRepresentation pr = new PortRepresentation(np, "NullPort");
    Socket s = null;
    
    System.out.println("Description is: " + pr.getDescription());
    System.out.println("Connected is: " + pr.getConnected());
    try {
      // get local host name
      InetAddress local = java.net.InetAddress.getLocalHost();
      String hn = local.getHostName();
      
      // open new server socket
      ServerSocket ss = new ServerSocket(0);
      int pn = ss.getLocalPort();
      
      pr.establishConnection(hn, pn);
      Socket s1 = ss.accept();
    }
    catch(Exception e) {
      System.out.println("PR: Exception: " + e);
    }
    System.out.println("Connected is: " + pr.getConnected());
  }
  */
}
