package webflow.backend;

import java.net.*;
import java.io.*;
import java.lang.*;

/**
* the SMrequest class is used when sending requests to the SessionManager 
* class. It extends the Message class. It stores three items in it: 
* - the command (one of an enumeration: NEW_MODULE, CONNECT, RUN, STOP, 
*   or DESTROY
* - two IDs: from and to. 
* - an integer: used to hold the module# (when creating a NEW_MODULE) 
* For the description of the communication protocol which SMrequest 
* implements look under communication protocols. 
*
*/
/* public for now. Should not be in the future */
public class SMrequest extends Message {
  /** 
  * an enumeration of available commands to be sent through a SMrequest
  */
  public static final int NEW_MODULE = 0, CONNECT = 1, RUN = 2, STOP = 3, 
			  DESTROY = 4 , DESTROY_MODULE = 5 , END_SESSION = 6 ;

  // SMrequest contains (at most): a name of module, a command to perform
  // two module IDs, and two port IDs
  String name1;
  int cmd;
  ID cfrom ,cto;
  ID mid1,mid2;


 /** 
  * void constructor used by the receiving side to create an empty 
  * SMrequest object to get the incomming message
  */
  public SMrequest() {
    cmd = 0;
    cfrom = cto = null;
    mid1 = mid2 = null; 
  }
   
  /** 
  * constructor used to implement the RUN, STOP, and DESTROY 
  * commands (that need only a command)
  * @param command - int: command requested
  */
  public SMrequest(int command) {
    cmd = command;
  }

  /**
  * constructor used to implement the NEW_MODULE command
  * @param command - int: command requested
  * @param which - int: which module to create
  */
  public SMrequest(int command, String  which) {
    cmd = command;
    name1 = which;
  }

  /**
  * constructor used to implement the CONNECT command
  * @param command - int: command requested
  * @param from - ID of the port to connect to 'to'
  * @param to - ID of the port to connect to 'from'
  * @param mID1 - ID of the module to whom 'from' belongs
  * @param mID2 - ID of the module to whom 'to' belongs
  */
  public SMrequest(int command, ID from, ID to ,ID mID1,ID mID2) {
    cmd = command;
    cfrom = from;
    cto = to;
    mid1 = mID1;
    mid2 = mID2;
  }

 
  /**
  * return the command stored in SMrequest
  * @return value of the command stored in SMrequest
  */
  public int getCommand() {
    return cmd;
  }

  /**
  * return the moduleNumber parameter stored in SMrequest 
  * @return int: value of the moduleNumber stored in the object
  */
  public String  getModuleName() {
    return name1;
  }

  /** 
  * return the from parameter stored in SMrequest
  * @return ID stored in the 'from' parameter in SMrequest
  */
  public ID getFrom() {
    return cfrom;
  }

  /**
  * return the to parameter stored in SMrequest
  * @return ID stored in the 'to' parameter in SMrequest
  */
  public ID getTo() {
    return cto;
  }

  /**
  * return the mID1 parameter stored in SMrequest
  * @return the Module ID stored in the mID1 parameter in SMrequest
  */
  public ID getModuleID1(){
    return mid1;
  }
  
  /**
  * return the mID2 parameter stored in SMrequest
  * @return the Module ID stored in the mID2 parameter in SMrequest
  */
  public ID getModuleID2(){
    return mid2;
  }

  /**
  * read the SMrequest from the input stream 'in'. 
  * @param in: an InputStream from which the representation for the 
    * 	       SMrequest will be read. 
  * @return true if the SMrequest was read, false if some error occured. 
  */
  public boolean fromStream(InputStream in) { 
    boolean done =true;
    try
      {       
	DataInputStream din = new DataInputStream(in);

	// item 1: int - command
	cmd = din.readInt();

	// item 2: String - module name, guarded for null
	if(din.readBoolean())
	  {
	    int bufsize= din.readInt();
	    byte[] b = new byte[bufsize];
	    din.readFully(b,0,bufsize);
	    name1 = new String(b,0,0,bufsize);
	  }
	else
	  name1 = null;

	// item 3: ID - 'from' port ID, guarded for null
	if (din.readBoolean())
	  {
	    cfrom = new ID();
	    cfrom.fromStream(in);
	  }
	else
	  cfrom = null;

	// item 4: ID - 'to' port ID, guarded for null
	if (din.readBoolean())
	  {
	    cto = new ID();
	    cto.fromStream(in);
	  }
	else
	  cto = null;
	
	// item 5: ID - 'from' module ID, guarded for null
	if (din.readBoolean())
	  {
	    mid1 = new ID();
	    mid1.fromStream(in);
	  }
	else
	  mid1 = null;

	// item 6: ID - 'to' module ID, guarded for null
	if (din.readBoolean())
	  {
	    mid2 = new ID();
	    mid2.fromStream(in);
	  }   
	else
	  mid2 = null;

	done=true;
      }
    catch (IOException e)
      {
	cfrom =cto = null; 
	mid1=mid2=null; 
	done = false;
      }
    
    return done;
  }
  
  /**
  * write the SMrequest to the output stream 'out'. 
  * @param out: an OutputStream to which the representation for the SMrequest
  * 		will be written. 
  * @return true if the SMrequest was written, false if some error occured. 
  */
  public boolean toStream(OutputStream out) {
    boolean done;
    try
      {
	DataOutputStream dout = new DataOutputStream(out);

	// item 1: int - command
        dout.writeInt(cmd);

	// item 2: String - module name, guarded for null
	if(name1 == null)
	  dout.writeBoolean(false);
	else {
	  dout.writeBoolean(true);
	  dout.writeInt(name1.length());
	  dout.writeBytes(name1);
	}

	// item 3: ID - 'from' port ID, guarded for null
        if(cfrom == null)
	  dout.writeBoolean(false);
        else 
	  { 
	    dout.writeBoolean(true); 
	    cfrom.toStream(dout);
	  }

	// item 4: ID - 'to' port ID, guarded for null
        if(cto == null)
	  dout.writeBoolean(false);
	else 
	  { 
	    dout.writeBoolean(true);
	    cto.toStream(dout);
	  }

	// item 5: ID - 'from' module ID, guarded for null
        if (mid1 ==null)
	  dout.writeBoolean(false);
	else 
	  {
            dout.writeBoolean(true); 
            mid1.toStream(dout);
	  }

	// item 6: ID - 'to' module ID, guarded for null
        if (mid2 ==null)
	  dout.writeBoolean(false);
        else 
	  { 
	    dout.writeBoolean(true);
	    mid2.toStream(dout);
	  }

	dout.flush();
	done=true;
      }
    catch(IOException e)
      {
	done=false;
      }
    
    return done;
  }

  /**
   * test stub
   */
  public static void main(String[] argv)
  {
    ID cfrom = new ID("blah1",1);
    ID cto = new ID("blah2" , 2);
    ID mid1 = new ID ("module #1",3);
    ID mid2 = new ID ("module #2",4);
    SMrequest req = new SMrequest(SMrequest.CONNECT,cfrom,cto,mid1,mid2);
    SMrequest req1 = new SMrequest(SMrequest.NEW_MODULE,"hello");
    if(argv[0].equals("get"))
      {
	System.out.println("cmd:" + req.getCommand());
	req.getModuleID1().toStream(System.out);
	System.out.println("at:");
	req.getFrom().toStream(System.out);
	System.out.println("to:"); 
	req.getModuleID2().toStream(System.out);
	System.out.println("at:"); 
	req.getTo().toStream(System.out);
	System.out.println("cmd:" + req1.getCommand());
      }
    else if(argv[0].equals("from"))
      {
	SMrequest request = new SMrequest();
	if(!request.fromStream(System.in))
          System.out.println("error  occured");
	else
	  request.toStream(System.out);
      }
    else if(argv[0].equals("to"))
      req.toStream(System.out);
    else if(argv[0].equals("to1"))
      req1.toStream(System.out);
  }
  
}



























