package webflow.backend;

import javax.servlet.*;
import java.net.*;
import java.util.*;
import java.io.*;

import webflow.frontend.*;

/**
 * SessionManager is a jeeves Servlet. It maintains a representation of
 * the data flow diagram, greated with the FrontEnd. It interprets user 
 * commands from the FrontEnd into mapping of the Modules on physical servers,
 * send initialize Module requests to the ModuleManager at appropriate server,
 * send initialize connection between ports to ConnectionManager, starts the
 * execution and  receives notification for the trafic in the created network. 
 * 
 * In the prototype, the SessionManager also doubles as a viewer manager. 
 * When viewer modules are created, they return a string containing HTML code
 * for loading up an applet. The SessionManager stores that string, and issues
 * a key towards it. The key is used by the FrontEnd to retrieve the applet 
 * code
 */
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it. 
*/
public class 
WFlowSession implements WebFlowDebugFlags 
{
  
  /** Hashtable of proxymodules indexed by the module number */
  Hashtable ProxyML; 
  
  /** Server socket on which the SessionManager will listen */
  /** for incoming requests                                 */
  ServerSocket listener;
  
  /** HostName and port number on which the SessionManager is listening */
  String hostName;
  int portNumber = -1;

  /** thread which will service the listener socket */
  Thread socketThread;

  /** buffer of viewer URLs */
  ViewerList viewerList;

  /** list of modules and hosts on which they exist */
  ModuleSourceList moduleList;

  /** url of the list of modules and hosts */
  String moduleUrl;

  /**  diagnostic object */
  Diagnostic diag;

  /** diagnostics on or off (true = on, false = off) */
  static final boolean DIAGNOSTICS_ACTIVE = true;

  /**
  * initialize the SessionManager. It initialize a thread that will respond 
  * to FrontEnd calls, initialize the data structure for the  network,
  * initialize the sockect for outgoing commands, and socket for incomming
  * notification
  * @param initialParameters : ServletStub that contains at least one 
  *                            parameter: moduleFileURL - a URL towards the
  *                            file of all modules
  */
  public
  WFlowSession(String _moduleUrl) 
  {
    /** the module file contains the descriptions, names, and */
    /** hosts on which the modules in the system reside       */

    moduleUrl = new String(_moduleUrl);

    if(DEBUG==true) 
       System.out.println("I am in WFlowSession moduleUrl :"+moduleUrl);

    /** Initialize ModuleList */
    moduleList = new ModuleSourceList();

    if(DEBUG==true) 
       System.out.println("moduleList is initialized");

    /** for now just ignore whether the parse method worked or not */
    moduleList.parse(moduleUrl);

    if(DEBUG==true) 
       System.out.println("moduleUrl is being parsed!");


    /** Initialize Module Hashtable */
    ProxyML = new Hashtable();

    /** Initialize Viewer List */
    viewerList = new ViewerList();

    if(DEBUG==true) 
       System.out.println("ViewerList and Hashtable is Initialized ");

    /** initialize the diagnostic object */
    diag = new Diagnostic("sm.diag");
    diag.activate(DIAGNOSTICS_ACTIVE);
    diag.write("Activated diagnostics object");

  }// end of constructor

  /**
  */
  public synchronized void 
  refreshModules()
  { 

     /** for now just ignore whether the parse method worked or not */
     moduleList.parse(moduleUrl);

     if(DEBUG==true) 
       System.out.println("moduleUrl is being parsed!");

  }// end of getPortNumber()

  /**
  * Gets the port number on which this session is listening
  * @return Port Number
  */
  public synchronized int 
  getPortNumber()
  { 
     return portNumber;

  }// end of getPortNumber()

  /**
  * Finds the HTML definition of the Viewer Object(Applet). 
  * @param reqKey : A key for the module
  * @return The HTML definition of the viewer object.
  */
  public synchronized String 
  getHTMLofViewer(String reqKey)
  {
    String ret = null;

    /** if it succeeded, get the string from viewerList,      */
    /** and put it to output otherwise leave the output blank */
    if(reqKey != null) 
    { int key = Integer.parseInt(reqKey);
      ret = viewerList.getViewer(key);
    }

    if(DEBUG==true)
      System.out.println((ret!=null?"Viewer HTML = " + ret:"ret is null"));

    return ret;

  }// end of getHTMLofViewer()


  /**
  * select one of the servers on the serverList
  * @param serverList - Vector containing a list of servers
  * @return String - one of the Strings in the Vector
  */
  public synchronized String 
  selectServer(Vector serverList) 
  {
    double random = Math.random();
    int serverListSize = serverList.size();
    int whichServer = (int)Math.round(random * (serverListSize - 1));
    String serverName = (String)serverList.elementAt(whichServer);

    return serverName;

  }// end of selectServer()

  /**
  * Finds the server name to run this module on.
  *
  * @param ModuleName - The name of the module
  * @return The Servername for this module
  *         When ther is no server, it will return <b>null</b>
  */ 
  public synchronized String
  getServerNameFor(String ModuleName) 
  { 
    String ret = null;

    diag.write("new module request received for module " + ModuleName);
    if(DEBUG==true)
        System.out.println("new module request received for module " + ModuleName);

    /** first get a list of servers that have module 'ModuleName' on them */
    Vector serverList = moduleList.findServer(ModuleName);

    /** and then select one of them */
    if(serverList != null)
      ret = new String(selectServer(serverList));

    diag.write("will initialize the module on server " + ret);
    if(DEBUG==true)
       System.out.println("will initialize the module on server " + ret);

    return ret;

  }// end of getServerNameFor()

  
  /**
  * Adds this module to the current session object.
  *
  * @return <b>0</b> htmlString <br>
  *         <b>1</b> portStorage  
  *         <b>3</b> htmlKey  
  */
  public Vector
  addModuleToTable(MetaModule mm, String hostName, String serverName,  
                   int MMportNumber, int CMportNumber)
  {
	/** if not, extract the MetaModule, ModuleID, the (possibly empty) */
	/** HTML string, and all the ports                                 */
	ID moduleID = mm.getModuleID();
	Vector portStorage = new Vector();
    int htmlKey = 0;
    String htmlString = null;

    if(mm.getHTML()!= null)
    //{  htmlString = new String("<P><APPLET CODEBASE =\""+"http://");
    {  htmlString = new String("<P><APPLET ");
       //htmlString += "NAME=\""+mm.getModuleID().getModuleName()+"\" ";
       htmlString += "NAME=\""+mm.getModuleID().getModuleName()+"\" ";
       htmlString += "CODEBASE =\""+"http://";
       htmlString += serverName+"/webflow\" ";
       htmlString+=mm.getHTML();
       if(DEBUG==true)
          System.out.println("String = "+htmlString);
    }// end of if

    /** extract  the ports */
    ID portID = mm.getFirstPortID();

    while(portID != null) 
    {
       portStorage.addElement(portID);
       portID = mm.getNextPortID();

    }// end of while
	  
    diag.write("making new module representation for the module");
    if(DEBUG==true)
       System.out.println("making new module representation for the module");

    /** make a module representation from the MetaModule */
    ModuleRepresentation moduleRep = new 
           ModuleRepresentation(hostName, mm, MMportNumber, CMportNumber);

    /** store the module representation */
    synchronized(ProxyML) { ProxyML.put(moduleID, moduleRep); }

    /** if the module needs an HTML called, store that HTML */
    if(htmlString != null) 
    {
	    synchronized(viewerList) { htmlKey = viewerList.addViewer(htmlString); };
	    diag.write("viewer module was returned. Key is " + htmlKey);

    }// end of if

    Vector ret = new Vector(3);
    ret.insertElementAt(htmlString,0);
    ret.insertElementAt(portStorage,1);
    ret.insertElementAt(new Integer(htmlKey),2);
    
    return ret ;

  }// end of addModuleToTable()
  
  /**
  * Gets the Diagnostic object of this session.
  * @return Diagnostic Object of this Session.
  */
  public synchronized Diagnostic 
  getDiagnostic()
  { return diag; }  

  /**
  * Returns the ModuleRepresentation of the Module denoted 
  * with ID.
  *
  * @param  _ID - Module ID Info. 
  * @return ModuleRepresentation  of the Module
  */
  public synchronized ModuleRepresentation
  getModuleRepWithID( ID _ID)
  { return (ModuleRepresentation) ProxyML.get(_ID); }

  /**
  * Removes the module, given with ID, from the module table.
  * @param _ID - Module ID Info.
  * @return <b>null</b> If the remove is unsuccessfull. <br>
  *         <b>not null</b> if it is successfull.
  */
  public synchronized Object 
  removeModuleWithID(ID _ID)
  { return ProxyML.remove(_ID); }

  /**
  * Gets the Module Tables
  * @return - Module Hashtable.
  */
  public synchronized Hashtable
  getModuleTable()
  {  return ProxyML; }

  /**
  * Deletes all the entries in the Module Hashtable.
  */
  public synchronized void
  disposeModuleTable()
  {  ProxyML.clear(); }

  /**
  * Destroys the object.
  */
  public void
  destroy()
  {
  }// end of destroy()

}// end of class WFlowSession
