package webflow.util.EventDispatcher;

import java.awt.EventQueue;
import webflow.util.*;

/**
 * WFEventQueue maintains a queue of events. Other components may post or
 * remove events from it. It is thread-safe, i.e. multiple threads may
 * access the queue simultaneously with no danger of race conditions. <BR>
 * Threads that call getEvent() when there are no events available, block, 
 * and are awakened by either threads that post an event, or threads that
 * have just been awakened (after being blocked waiting for an event) which
 * have noticed that there are more events available on the queue. 
 *
 * @version 0.1.0 first draft, April 25, 1997
 */
public class 
WFEventQueue extends Queue 
{

  /** number of threads waiting for an event */
  int numberWaiting;

  /**
  * Creates a WFEventQueue. The queue will be unusable until properly 
  * initialized. 
  */
  public 
  WFEventQueue() 
  {
    super();
    numberWaiting = 0;

  }// end of constructor

  /**
  * Returns the name of the component. 
  * @return "WFEventQueue" - the name of WFNEventQueue
  */
  public String 
  getComponentName() 
  {
    return("WFEventQueue");

  }// end of getComponentName()

  public String
  toString()
  { return getComponentName(); }

  /**
  * Posts an event on the event queue. If there are threads waiting to get
  * an event, wakes one of them. 
  * @param event the event to post
  */
  public synchronized void 
  postEvent(WFEvent event) 
  {
    /************************/
    /** 1. add to the queue */
    /************************/

    QueueItem item = new QueueItem(event);

    try 
    { addItem(item);
    } catch(NotAllowableQueueItemException naqie)
      {  System.out.println(" Error :"+naqie.getMessage()); }

    /*****************************************/
    /** 2. if number of waiting threads > 0, */
    /**    notify one of them                */
    /*****************************************/

    if(numberWaiting > 0) notify();

  }// end of postEvent()

  /**
  * Gets an event from the event queue. If there are no events available, 
  * a call to this method blocks until an event becomes available. 
  * @return the event on the head of the queue <BR>
  *  if not initialized, return null
  */
  public synchronized WFEvent 
  getEvent() 
  {
    WFEvent rett = null;

    /*******************************************/
    /** 1. if there are no events on the queue */
    /*******************************************/

    if(isEmpty())
    {
       /********************************************/
       /** 1.1. increase number of waiting threads */
       /********************************************/

       numberWaiting++;

       /*************************************************************/
       /** 1.2. wait for notification (give up lock on this object) */
       /** in case of interruption, make sure not to read from      */
       /** an empty head                                            */
       /*************************************************************/

       boolean emptyHead = true;

       while(emptyHead) 
       {

         try 
         { wait();
         } /*****************************************/
           /** if an interruption occurs, ignore it */
           /*****************************************/
           catch(InterruptedException ie) { }

         /******************************************/
         /** in any case: notified or interrupted, */
         /** don't leave the loop                  */
         /** until an event has been posted there  */
         /******************************************/

         emptyHead = isEmpty();

       }// end of while

       /************************************************/
       /** 1.3. decrease the number of waiting threads */
       /************************************************/

	   numberWaiting--;

    } // if Queue is EMPTY
 
    /***********************************************/
    /** 2. get the event on the head of the queue  */
    /** and remove it from the head                */
    /***********************************************/

    try
    { 
       QueueItem eQItem = getItem();

       rett = (WFEvent) eQItem.getObject();

    } catch (QueueEmptyException qee) { rett = null ; }

    /****************************************/
    /** 3. if number of waiting threads > 0 */
    /** 3.1 if there are more events on the */ 
    /** queue, notify                       */
    /****************************************/
      
    if( ( numberWaiting > 0 ) && ( isEmpty() == false ) ) notify();

    return rett;

  }// end of getEvent()

}// end of class WFNEventQueue

