package webflow.util.EventDispatcher;

import webflow.util.Action;
import webflow.util.Workers.*;
import java.util.Vector;
import java.util.Enumeration;

/**
 * WFEventQueueMonitor is the class that monitors the WFEventQueue. Whenever an 
 * event occurs on the queue, it checks to see which Listeners are registered
 * with it, and invokes them. <BR>
 * Generally speaking, they should be invoked in separate threads, but since 
 * the majority of listeners will just ignore the event, they can just be
 * invoked sequentially. However, this means writing the listeners in such 
 * a way that if they need to perform an action, they perform it in a separate 
 * thread. <BR>
 * It is implemented in a thread-safe way, i.e. one thread can register 
 * listeners, while another one posts events, and a third one runs over the
 * queue to look at events. 
 * 
 * @version 0.1.0 first draft, April 25, 1997
 */
public class 
WFEventQueueMonitor extends Object 
                    implements Action 
{
  /** queue monitored */
  WFEventQueue queue = null;

  /** list of event listeners */
  Vector listeners = null;

  /** My WorkerPoolManager Object */
  WorkerPoolManager WPool = null;

  /**
  * Creates an WFEventQueueMonitor which will monitor the event queue. 
  */
  public 
  WFEventQueueMonitor() 
  {
    queue = new WFEventQueue();
    listeners = new Vector();

    WPool = new WorkerPoolManager();

  }// end of constructor

  /**
  * Returns the name of the component. 
  * @return "WFEventQueueMonitor" - the name of WFEventQueueMonitor
  */
  public String 
  getComponentName() 
  {
    return("WFEventQueueMonitor");
  }

  public String
  toString()
  { return getComponentName(); }

  /**
  * Keeps checking the event queue for events, and invoking the registered
  * listeners. Action Interface.
  */
  public void 
  service() 
  {
     while(true) 
     {
       /***********************************************/ 
       /** if any exception occurs inside the loop,   */
       /** deal with it, and repeat the loop, forever */
       /***********************************************/ 

	   try 
       {
          /*************************************/ 
          /** 1. get the next event, and block */
          /**    if no events available        */
          /*************************************/ 

	      WFEvent event = queue.getEvent();

          /************************************************/
          /** 2. get the list of listeners (synchronized) */
          /************************************************/

	      Enumeration enum;

          /*******************************************************/
          /** since Vector is a thread-safe class, synchronizing */
          /** over it explicitly is not needeed now. However,    */
          /** further developments might require synchronization,*/
          /** so it is better to just leave it here              */
          /*******************************************************/

          synchronized(this) 
          {
             enum = listeners.elements();
          }

          /*******************************************************/
          /** 3. for each listener, invoke its eventOccured()    */
          /**    method in version 0.1.0, invoke all listeners   */
          /**    one after another. maybe in next versions,      */
          /**    start workers that will invoke them concurently.*/ 
          /*******************************************************/

          while(enum.hasMoreElements()) 
              WPool.put( new WFEventNotification((WFEventListener) enum.nextElement(), event) );
          
          /**********************/
          /** 4. repeat forever */
          /**********************/

       } /**************************************************/
         /** class cast exception will occur when the next */
         /** event on the queue is not a WebFlowEvent      */
         /**************************************************/
	     catch(java.lang.ClassCastException cce) {
	       // do nothing
         }

     }// end of while

  }// end of service()

  /**
  * Adds a listener for WebFlowEvents to the list of listeners. 
  * Each of these listeners will be invoked when an event occurs. 
  */
  public void 
  addListener(WFEventListener listener) 
  {
     /**********************************************************/
     /** all accesses to Vector listeners are synchronized     */
     /** actually, since the listeners are stored in a Vector, */ 
     /** which is a thread-safe class, this is unnecessary.    */
     /** However, it ilustrates that listeners can be added    */
     /** in a thread-safe fashion, and should be               */
     /** left here. Especially if code is further added.       */
     /**********************************************************/
     synchronized(this) 
     {
        listeners.addElement(listener);
     }

  }// end of addListener()

}// end of EventQueueMonitor
