package webflow.util.Workers;

import webflow.util.Action;

/**
 * ActionStorage stores a reference to an Action. ActionStorage allows the
 * Action to be inserted and retrieved. It also allows for checking for 
 * the presence of an Action. 
 * All these operations are performed in a thread-safe 
 * way. <BR>
 * The main way in which this class will concievably be used is by Worker
 * objects, to ensure that no race condition occurs when two threads store 
 * and retrieve a reference to an Action 
 *
 * @version 0.1.0 first draft, April 20, 1997
 */
public class ActionStorage extends Object {
  // action stored in this storage
  Action storedAction;

  /**
   * Constructs an empty ActionStorage object. getAction() returns null, 
   * and checkForAction() returns false on such an object before setAction() 
   * is invoked. 
   */
  public ActionStorage() {
    // no action stored at this point in time
    storedAction = null;
  }

  /**
   * Sets the Action stored in this object. 
   * @param a the Action to be stored in this object
   */
  public synchronized void setAction(Action a) {
    // just set the action to a. No race conditions possible, since
    // this method is synchronized
    storedAction = a;
  }

  /**
   * Returns the Action stored in this object. <BR>
   * After this method is called, subsequent calls to getAction() will return
   * null, and subsequent calls to checkForAction() will return false, until 
   * an Action has been stored with setAction(). 
   * @return the Action stored
   */
  public synchronized Action getAction() {
    // temporary copy of the stored action, since
    Action toReturn = storedAction;

    // the action stored will be deleted
    storedAction = null;

    // return the stored copy
    return toReturn;
  }

  /**
   * Checks whether this object stores an Action or not. 
   * @return true, if the Action has been set <BR>
   *  false, otherwise
   */
  public synchronized boolean checkForAction() {
    // if an action has been stored, storedAction will equal non-null, and
    // this method will return true. Otherwise, it will return false
    return (storedAction != null);
  }
}
